/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JadortCommand.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.client.console;

import java.io.File;
import java.net.URL;
import java.util.List;

import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean;
import org.ow2.jasmine.jadort.service.entities.GroupBean;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean.Step;
import org.ow2.jasmine.jadort.service.IJadortService;

/**
 * Interface to call method on JaDOrT service
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public class JadortCommand {

    private IJadortService JaDOrT;

    /**
     * Constructor
     * 
     * @param JaDOrT The instance of interface of JaDOrT service
     */
    public JadortCommand(final IJadortService JaDOrT) {
        this.JaDOrT = JaDOrT;
    }

    /**
     * Create a nex toperation ang go to the next operation step
     * 
     * @param newOperationName Name of the operation to create
     * @return The created operation
     * @throws Exception
     */
    public OperationStateBean createNewOperation(final String newOperationName) throws Exception {
        JaDOrT.createNewOperation(newOperationName);
        JaDOrT.next();
        return JaDOrT.getCurrentOperation();

    }

    /**
     * Return the operation list
     * 
     * @return The operation list
     * @throws Exception
     */
    public List<OperationStateBean> getOperationsList() throws Exception {
        return JaDOrT.getOperationsList();
    }

    /**
     * Select an operation on the JaDOrT service
     * 
     * @param operation The operation to select
     * @throws Exception
     */
    public void selectOperation(final OperationStateBean operation) throws Exception {
        JaDOrT.selectOperation(operation);
    }

    /**
     * Delete an operation
     * 
     * @param operation the operation to delete
     * @throws Exception
     */
    public void deleteOperation(final OperationStateBean operation) throws Exception {
        JaDOrT.deleteOperation(operation);
    }

    /**
     * Load a topology file
     * 
     * @param xmlTopoFile xml topology file to load
     * @throws Exception
     */
    public void loadTopology(final File xmlTopoFile) throws Exception {
        JaDOrT.loadTopology(xmlTopoFile);
        JaDOrT.next();
    }

    /**
     * Get the groups
     * 
     * @return the groups list
     * @throws Exception
     */
    public List<GroupBean> getGroups() throws Exception {
        return JaDOrT.getGroups();
    }

    /**
     * Select a group
     * 
     * @param selectedGroup the group to select
     * @throws Exception
     */
    public void selectGroup(final GroupBean selectedGroup) throws Exception {
        JaDOrT.selectGroup(selectedGroup);
        JaDOrT.next();
    }

    /**
     * Return the selected group
     * 
     * @return the selected group
     * @throws Exception
     */
    public GroupBean getSelectedGroup() throws Exception {
        return JaDOrT.getSelectedGroup();
    }

    /**
     * Create a application from an archive
     * 
     * @param url url of the archive
     * @return the created application
     * @throws Exception
     */
    public ApplicationBean createApplication(final URL url) throws Exception {
        ApplicationBean ret = JaDOrT.createApplication(url);
        JaDOrT.next();

        return ret;
    }

    /**
     * Return the current operation step
     * 
     * @return the current operation step
     * @throws Exception
     */
    public Step getCurrentStep() throws Exception {
        return JaDOrT.getCurrentStep();
    }

    /**
     * ask the service if it is ready to go to the next step
     * 
     * @return true or false
     * @throws Exception
     */
    public boolean canGoToNextStep() throws Exception {
        return JaDOrT.canGoToNextStep();
    }

    /**
     * Go to the next step
     * 
     * @throws Exception
     */
    public void next() throws Exception {
        JaDOrT.next();
    }

    /**
     * ask the service if it is ready to go to the previous step
     * 
     * @return true or false
     * @throws Exception
     */
    public boolean canGoToPreviousStep() throws Exception {
        return JaDOrT.canGoToPreviousStep();
    }

    /**
     * Go to the previous step
     * 
     * @throws Exception
     */
    public void previous() throws Exception {
        JaDOrT.previous();
    }

    /**
     * Return the list of the deployments
     * 
     * @return the list of the deployments
     * @throws Exception
     */
    public List<DeploymentBean> getDeployments() throws Exception {
        return JaDOrT.getDeployments();
    }

    /**
     * Restart the last tried deplyoment operation on a server. To use when the
     * last operation failed.
     * 
     * @param server the server to use
     * @throws Exception
     */
    public void restartServer(final ServerBean server) throws Exception {
        JaDOrT.restartServer(server);
    }

    /**
     * Ignore the error state of a server. To use when server on error and after
     * user correct himself the error.
     * 
     * @param server the server to use
     * @throws Exception
     */
    public void ignoreServer(final ServerBean server) throws Exception {
        JaDOrT.ignoreServer(server);
    }

    /**
     * Check if the server is ready to go to the next step.
     * 
     * @param server the server to use
     * @return true or false
     * @throws Exception
     */
    public boolean checkServer(final ServerBean server) throws Exception {
        return JaDOrT.checkServer(server);
    }

}
