/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: IJadortService.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service;

import java.io.File;
import java.net.URL;
import java.util.List;

import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean;
import org.ow2.jasmine.jadort.service.entities.GroupBean;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;

/**
 * IJadortService interface defines all methods used for the deployment
 * orchestration
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public interface IJadortService {

    /**
     * Returns the current Step of the current operation
     *
     * @return The current step.
     */
    OperationStateBean.Step getCurrentStep() throws Exception;

    /**
     * Returns the list of operations
     *
     * @return List of Operations.
     */
    List<OperationStateBean> getOperationsList() throws Exception;

    /**
     * Delete an Operation from the operation list
     *
     * @param operation - the operation to be deleted
     */
    void deleteOperation(final OperationStateBean operation) throws Exception;

    /**
     * Select an Operation from the operation list
     *
     * @param operation - the operation to be selected
     */
    void selectOperation(final OperationStateBean operation) throws Exception;

    /**
     * Create a new operation
     *
     * @param newOperationName - the name of the new operation that will be
     *        created
     */
    void createNewOperation(final String newOperationName) throws Exception;

    /**
     * Returns the current operation
     *
     * @return The current operation.
     */
    OperationStateBean getCurrentOperation() throws Exception;

    /**
     * Tests if it is allowed to go to the next step
     *
     * @return true id calling {@link IJadortService#next()} is allowed,
     *         false otherwise.
     */
    boolean canGoToNextStep() throws Exception;

    /**
     * goes to next step
     *
     * @see IJadortService#canGoToNextStep()
     */
    void next() throws Exception;

    /**
     * Tests if it is allowed to go the previous step
     *
     * @return true id calling {@link IJadortService#previous()} is allowed,
     *         false otherwise.
     */
    boolean canGoToPreviousStep() throws Exception;

    /**
     * Goes back to the previous step.
     *
     * @see IJadortService#canGoToPreviousStep()
     */
    void previous() throws Exception;

    /**
     * Loads the topology from a .xml file.
     *
     * @param xmlTopoFile - the file path of the .xml file uploaded by the user,
     *        this file contains the topology description
     */
    void loadTopology(final File xmlTopoFile) throws Exception;

    /**
     * Returns the list of groups obtained from the loaded topology in the
     * current operation
     *
     * @return The list of groups
     */
    List<GroupBean> getGroups() throws Exception;

    /**
     * Select a group from the group list
     *
     * @param selectedGroup - the group to be selected
     */
    void selectGroup(final GroupBean selectedGroup) throws Exception;

    /**
     * Returns the list of server deployments in the current operation
     *
     * @return The list of deployment
     */
    List<DeploymentBean> getDeployments() throws Exception;

    /**
     * Returns the selected Group in the current operation
     *
     * @return the selected group
     */
    GroupBean getSelectedGroup() throws Exception;

    /**
     * This function allows to restart a task on a server (the server will
     * restart the last task)
     *
     * @param server - the server on which the task will be restarted
     */
    void restartServer(final ServerBean server) throws Exception;

    /**
     * this function checks if no probleme on the server and all problems are
     * resolved on the server.
     *
     * @return true-> OK the problem server is resolved and it can be ignored ,
     *         false -> KO the problem still
     * @param server - the server that will be checked
     * @see IJadortService#ignoreServer()
     */
    boolean checkServer(final ServerBean server) throws Exception;

    /**
     * this function forces the server to be ignored and continues this task
     *
     * @param server - the server that will be ignored
     */
    void ignoreServer(final ServerBean server) throws Exception;

    /**
     * this function forces the server to be aborted and continues this task
     *
     * @param server - the server that will be aborted
     */
    void abortServer(final ServerBean server) throws Exception;

    /**
     * Create a new application
     *
     * @return ApplicationBean - the created ApplicationBean
     * @param url - URL where to fetch the application. Can be local, remote
     *        (HTTP, FTP, ...)
     */
    ApplicationBean createApplication(final URL url) throws Exception;
}
