/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DummyServerAction.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.action;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;

/**
 * Dummy server action, used for testing the service.
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public class DummyServerAction extends ServerAction {

    protected String name;
    protected String url;
    protected String username;
    protected String password;

    private static int nonCrashProba = 80;

    private static Random random = new Random();

    enum State {
        PRESENT, DEPLOYED
    };

    enum Policy {
        DEFAULT, RESERVED, DISABLED, PRIVATE
    };

    class ApplicationInstanceInformation {
        private String name;
        private String version;
        private State state;
        private Policy policy;
        private List<String> mbeans = new ArrayList<String>();

        public String getName() {
            return this.name;
        }

        public void setName(final String name) {
            this.name = name;
        }

        public String getVersion() {
            return this.version;
        }

        public void setVersion(final String version) {
            this.version = version;
        }

        public State getState() {
            return this.state;
        }

        public void setState(final State state) {
            this.state = state;
        }

        public Policy getPolicy() {
            return this.policy;
        }

        public void setPolicy(final Policy policy) {
            this.policy = policy;
        }

        public List<String> getMbeans() {
            return this.mbeans;
        }

        public void setMbeans(final List<String> mbeans) {
            this.mbeans = mbeans;
        }
    }

    private Map<String, ApplicationInstanceInformation> applications = new Hashtable<String, ApplicationInstanceInformation>();

    private String dummyAppName(final String application) {
        return "/" + name + "/apps/" + application + ".ear";
    }

    private void addDummyApplication(final String name, final State state) {
        addDummyApplication(name, state, null, null);
    }

    private void addDummyApplication(final String name, final State state, final String version, final Policy policy) {
        String appName;
        if (version == null) {
            appName = dummyAppName(name);
        } else {
            appName = dummyAppName(name + "-version" + version);
        }
        ApplicationInstanceInformation appInfo = new ApplicationInstanceInformation();
        appInfo.setName(name);
        appInfo.setVersion(version);
        appInfo.setState(state);
        appInfo.setPolicy(policy);
        applications.put(appName, appInfo);
    }

    protected DummyServerAction(final ServerBean server) {
        this.name = server.getName();
        this.url = server.getConnector().getConnectorUrl();
        this.username = server.getConnector().getUsername();
        this.password = server.getConnector().getPassword();
        appendToLog("Created DummyServerAction for server '" + name + "'");

        addDummyApplication("application1", State.DEPLOYED, "V1", Policy.DISABLED);
        addDummyApplication("application1", State.DEPLOYED, "V2", Policy.DEFAULT);
        addDummyApplication("application2", State.DEPLOYED);
    }

    @Override
    public List<ApplicationBean> listOfApplications() {
        List<ApplicationBean> result = new ArrayList<ApplicationBean>(applications.size());
        for (ApplicationInstanceInformation appInfo : applications.values()) {
            ApplicationBean application = new ApplicationBean(appInfo.getName());
            if (appInfo.getState() == State.PRESENT) {
                application.setState(ServerAction.STATE_PRESENT);
            } else {
                application.setState(ServerAction.STATE_DEPLOYED);
            }
            if (appInfo.getVersion() != null) {
                application.setVersion(appInfo.getVersion());
                String policy = appInfo.getPolicy().toString();
                policy = policy.charAt(0) + policy.substring(1).toLowerCase();
                application.setPolicy(policy);
            }
            result.add(application);
        }
        return result;
    }

    /**
     * Checks that the state of an application is as expected, throws an
     * IllegalArgumentException otherwise.
     * 
     * @param appName Application name to check.
     * @param state State to check against, if null then the non-presence of
     *        appName is checked.
     * @return The ApplicationInstanceInformation for appName if state is as
     *         expected.
     */
    private ApplicationInstanceInformation checkState(final String appName, final State state) {
        if (appName == null) {
            throw new NullPointerException("Application is null");
        }
        ApplicationInstanceInformation appInfo = applications.get(appName);
        if (state == null) {
            if (appInfo != null) {
                throw new IllegalArgumentException("Application '" + appName + "' is " + appInfo.getState().toString()
                        + " whereas it should not exist");
            } else {
                return null;
            }
        } else {
            if (appInfo == null) {
                throw new IllegalArgumentException("Application '" + appName + "' does not exist");
            } else if (!state.equals(appInfo.getState())) {
                throw new IllegalArgumentException("Application '" + appName + "' is " + appInfo.getState().toString()
                        + " whereas it should be " + state.toString());
            } else {
                return appInfo;
            }
        }
    }

    /**
     * Checks that the state and policy of an application is as expected, throws
     * an IllegalArgumentException otherwise.
     * 
     * @param appName Application name to check.
     * @param state State to check against, if null then the non-presence of
     *        appName is checked.
     * @param policies Array of Policy to check against, the application's
     *        policy must match one of the elements of this array.
     * @return The ApplicationInstanceInformation for appName if state and
     *         policy are as expected.
     */
    private ApplicationInstanceInformation checkPolicy(final String appName, final State state, final Policy[] policies) {
        ApplicationInstanceInformation appInfo = checkState(appName, state);
        if (policies == null) {
            if (appInfo.getPolicy() == null) {
                return appInfo;
            } else {
                throw new IllegalArgumentException("Application '" + appName + "' has policy " + appInfo.getPolicy().toString()
                        + " whereas it shouldn't have any");
            }
        } else {
            if (appInfo.getPolicy() == null) {
                throw new IllegalArgumentException("Application '" + appName + "' is not versioned (its policy is null)");
            } else {
                for (Policy policy : policies) {
                    if (appInfo.getPolicy().equals(policy)) {
                        return appInfo;
                    }
                }
                throw new IllegalArgumentException("Application '" + appName + "' has policy " + appInfo.getPolicy().toString());
            }
        }
    }

    @Override
    public String upload(final ApplicationBean application) throws Exception {
        appendToLog("Starting upload of application '" + application.toString() + "'");

        long wait = DummyServerAction.random.nextInt(2000) + 1000;
        int crash = DummyServerAction.random.nextInt(100);
        appendToLog("This should take " + wait + " milliseconds and has " + crash + "% probability of crashing");

        String appName = dummyAppName(application.toString());
        checkState(appName, null);

        Thread.sleep(wait);
        if (crash > DummyServerAction.nonCrashProba) {
            throw new Exception("This has crashed for no reason !");
        }

        ApplicationInstanceInformation appInfo = new ApplicationInstanceInformation();
        appInfo.setName(application.getName());
        appInfo.setVersion(application.getVersion());
        appInfo.setState(State.PRESENT);
        applications.put(appName, appInfo);

        appendToLog("Upload of application '" + application.toString() + "' seems to be OK, now checking");
        checkState(appName, State.PRESENT);

        appendToLog("Application '" + application.toString() + "' uploaded as '" + appName + "'");
        return appName;
    }

    @Override
    public void deploy(final String appName) throws Exception {
        appendToLog("Starting deploy of application '" + appName + "'");

        long wait = DummyServerAction.random.nextInt(2000) + 1000;
        int crash = DummyServerAction.random.nextInt(100);
        appendToLog("This should take " + wait + " milliseconds and has " + crash + "% probability of crashing");

        ApplicationInstanceInformation appInfo = checkState(appName, State.PRESENT);

        Thread.sleep(wait);
        if (crash > DummyServerAction.nonCrashProba) {
            throw new Exception("This has crashed for no reason !");
        }

        appInfo.setState(State.DEPLOYED);
        if (appInfo.getVersion() != null) {
            appInfo.setPolicy(Policy.RESERVED);
        }

        appendToLog("Deploy of application '" + appName + "' seems to be OK, now checking");
        checkState(appName, State.DEPLOYED);

        appendToLog("Application '" + appName + "' now " + appInfo.getState().toString());
    }

    @Override
    public String setDefault(final String appName) throws Exception {
        appendToLog("Starting setting application '" + appName + "' as default");

        long wait = DummyServerAction.random.nextInt(2000) + 1000;
        int crash = DummyServerAction.random.nextInt(100);
        appendToLog("This should take " + wait + " milliseconds and has " + crash + "% probability of crashing");

        ApplicationInstanceInformation appInfo = checkPolicy(appName, State.DEPLOYED, new Policy[] { Policy.RESERVED,
                Policy.DISABLED, Policy.PRIVATE });

        Thread.sleep(wait);
        if (crash > DummyServerAction.nonCrashProba) {
            throw new Exception("This has crashed for no reason !");
        }

        String oldDefaultName = null;
        for (Map.Entry<String, ApplicationInstanceInformation> entry : applications.entrySet()) {
            if (entry.getValue().getName().equals(appInfo.getName()) && entry.getValue().getPolicy().equals(Policy.DEFAULT)) {
                entry.getValue().setPolicy(Policy.DISABLED);
                oldDefaultName = entry.getKey();
                appendToLog("Application '" + oldDefaultName + "' now has policy " + entry.getValue().getPolicy().toString());
                break;
            }
        }
        appInfo.setPolicy(Policy.DEFAULT);

        appendToLog("Set application '" + appName + "' as default seems to be OK, now checking");
        checkPolicy(appName, State.DEPLOYED, new Policy[] { Policy.DEFAULT });

        appendToLog("Application '" + appName + "' now has policy " + appInfo.getPolicy().toString());
        return oldDefaultName;
    }

    @Override
    public void undeploy(final String appName) throws Exception {
        appendToLog("Starting undeploy of application '" + appName + "'");

        long wait = DummyServerAction.random.nextInt(2000) + 1000;
        int crash = DummyServerAction.random.nextInt(100);
        appendToLog("This should take " + wait + " milliseconds and has " + crash + "% probability of crashing");

        ApplicationInstanceInformation appInfo = checkState(appName, State.DEPLOYED);

        Thread.sleep(wait);
        if (crash > DummyServerAction.nonCrashProba) {
            throw new Exception("This has crashed for no reason !");
        }

        appInfo.setState(State.PRESENT);
        if (appInfo.getVersion() != null) {
            appInfo.setPolicy(null);
        }

        appendToLog("Undeploy of application '" + appName + "' seems to be OK, now checking");
        checkState(appName, State.PRESENT);

        appendToLog("Application '" + appName + "' now " + appInfo.getState().toString());
    }

    @Override
    public void erase(final String appName) throws Exception {
        appendToLog("Starting erase of application '" + appName + "'");

        long wait = DummyServerAction.random.nextInt(2000) + 1000;
        int crash = DummyServerAction.random.nextInt(100);
        appendToLog("This should take " + wait + " milliseconds and has " + crash + "% probability of crashing");

        checkState(appName, State.PRESENT);

        Thread.sleep(wait);
        if (crash > DummyServerAction.nonCrashProba) {
            throw new Exception("This has crashed for no reason !");
        }

        applications.remove(appName);

        appendToLog("Erase of application '" + appName + "' seems to be OK, now checking");
        checkState(appName, null);

        appendToLog("Application '" + appName + "' erased");
    }

    @Override
    public ApplicationBean getApplicationBean(final String appName) throws Exception {
        appendToLog("Getting application bean for application '" + appName + "'");

        if (appName == null) {
            return null;
        }

        ApplicationInstanceInformation appInfo = applications.get(appName);
        if (appInfo == null) {
            return null;
        }

        ApplicationBean application = new ApplicationBean(appInfo.getName());
        if (appInfo.getState() == State.PRESENT) {
            application.setState(ServerAction.STATE_PRESENT);
        } else {
            application.setState(ServerAction.STATE_DEPLOYED);
        }
        if (appInfo.getVersion() != null) {
            application.setVersion(appInfo.getVersion());
            String policy = appInfo.getPolicy().toString();
            policy = policy.charAt(0) + policy.substring(1).toLowerCase();
            application.setPolicy(policy);
        }
        return application;
    }
}