/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JonasServerAction.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.action;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;

/**
 * Action for the JOnAS 5 server.
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public class JonasServerAction extends ServerAction {

    private static final int BUFFER_SIZE = 1024;

    protected String name;
    protected String url;
    protected String username;
    protected String password;

    protected ObjectName j2eeServer;

    /**
     * Connection to the MBean server.
     */
    private MBeanServerConnection mbscnx = null;

    protected class VersionInformation {
        public String policy;
        public ObjectName mbean;

        public VersionInformation(final String policy, final ObjectName mbean) {
            this.policy = policy;
            this.mbean = mbean;
        }
    }

    protected class ApplicationInformation {
        public Map<String, VersionInformation> versions;
        public Set<ObjectName> managers;

        public ApplicationInformation() {
            versions = new HashMap<String, VersionInformation>();
            managers = new HashSet<ObjectName>();
        }
    }

    protected JonasServerAction(final ServerBean server) {
        this.name = server.getName();
        this.url = server.getConnector().getConnectorUrl();
        this.username = server.getConnector().getUsername();
        this.password = server.getConnector().getPassword();
        appendToLog("Created JonasServerAction for server '" + name + "'");
    }

    /**
     * Checks connectivity with the JMX URL.
     */
    @SuppressWarnings("unchecked")
    protected void checkJMXConnection() throws IOException, MalformedObjectNameException {
        if (mbscnx != null) {
            try {
                mbscnx.getMBeanCount();
            } catch (IOException e) {
                mbscnx = null;
                appendToLog("Connection dropped, reconnecting to JMX server on URL '" + this.url + "'");
            }
        }

        if (mbscnx == null) {
            appendToLog("Trying to connect to JMX server on URL '" + this.url + "'");
            Map<String, Object> env = null;
            if (this.username != null && this.password != null) {
                env = new HashMap<String, Object>(1);
                String[] creds = { this.username, this.password };
                env.put(JMXConnector.CREDENTIALS, creds);
            }
            JMXConnector c = JMXConnectorFactory.connect(new JMXServiceURL(this.url), env);
            mbscnx = c.getMBeanServerConnection();
            j2eeServer = ((Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=J2EEServer,*"), null)).iterator().next();
            appendToLog("JMX server connection OK for server '" + name + "', J2EEServer is '" + j2eeServer + "'");
        }
    }

    /**
     * Gets the list of deployed applications from the server.
     *
     * @return Map of applications. The map should be read the following way:
     *         <ul>
     *         <li>Keys are the module name (EAR or standalone)
     *         <li>Values are the version details for the corresponding
     *         versioned module (map of versions + policy as well as all MBeans
     *         used for switching between versions). If the module is not
     *         versioned, its value is null.
     *         </ul>
     */
    @SuppressWarnings("unchecked")
    protected Map<String, ApplicationInformation> getApplicationsList() throws Exception {
        // TODO: a much easier (and much more robust) way of doing this is to
        // use the JaDOrT service's JMX API for getting the version
        // identifier and the JNDI prefix. Only problem: that has only been
        // made available since JOnAS 5.0.4
        Map<String, ApplicationInformation> applications = new HashMap<String, ApplicationInformation>();

        // Non-versioned EJB-JARs and WARs, including those that are in EARs.
        Set<ObjectName> nonVersionedEjbsAndWars = new HashSet<ObjectName>();

        // Key = real ObjectName, Value = manager ObjectName. This therefore
        // implies that there'll be multiple keys pointing to the same value if
        // there are multiple versions of an application that're deployed.
        Map<ObjectName, ObjectName> versionedEjbsAndWars = new HashMap<ObjectName, ObjectName>();

        // Get the list of all deployed applications, use the obtained
        // information in order to fill nonVersionedEjbsAndWars and
        // versionedEjbsAndWars.
        {
            Set<ObjectName> realEjbMBeans = new HashSet<ObjectName>();
            Set<ObjectName> virtualEjbMBeans = new HashSet<ObjectName>();
            for (ObjectName ejbjar : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=EJBModule,*"), null)) {
                if ("true".equals(ejbjar.getKeyProperty("virtualContext"))) {
                    virtualEjbMBeans.add(ejbjar);
                } else {
                    realEjbMBeans.add(ejbjar);
                }
            }
            for (ObjectName virtualEjbMBean : virtualEjbMBeans) {
                String baseName = virtualEjbMBean.getKeyProperty("name").substring("VirtualContainer-".length());
                for (ObjectName ejbMBean : realEjbMBeans) {
                    // If the EJB-JAR is in an EAR, then the JNDI prefix will
                    // be set using the EAR's (J2EEApplication) name. Therefore
                    // check for the naming base in both attributes.
                    if (ejbMBean.getKeyProperty("name").startsWith(baseName)) {
                        versionedEjbsAndWars.put(ejbMBean, virtualEjbMBean);
                    } else {
                        String appName = ejbMBean.getKeyProperty("J2EEApplication");
                        if (appName != null && !appName.equals("none") && appName.startsWith(baseName)) {
                            versionedEjbsAndWars.put(ejbMBean, virtualEjbMBean);
                        }
                    }
                }
            }

            Set<ObjectName> realWarMBeans = new HashSet<ObjectName>();
            Set<ObjectName> virtualWarMBeans = new HashSet<ObjectName>();
            for (ObjectName war : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=WebModule,*"), null)) {
                if ("true".equals(war.getKeyProperty("virtualContext"))) {
                    virtualWarMBeans.add(war);
                } else {
                    realWarMBeans.add(war);
                }
            }
            for (ObjectName virtualWarMBean : virtualWarMBeans) {
                String baseName = virtualWarMBean.getKeyProperty("name");
                for (ObjectName warMBean : realWarMBeans) {
                    if (warMBean.getKeyProperty("name").startsWith(baseName)) {
                        versionedEjbsAndWars.put(warMBean, virtualWarMBean);
                    }
                }
            }
            realEjbMBeans.removeAll(versionedEjbsAndWars.keySet());
            realWarMBeans.removeAll(versionedEjbsAndWars.keySet());
            nonVersionedEjbsAndWars.addAll(realEjbMBeans);
            nonVersionedEjbsAndWars.addAll(realWarMBeans);
        }

        // Process all versioned items
        for (Map.Entry<ObjectName, ObjectName> versionedEjbOrWar : versionedEjbsAndWars.entrySet()) {
            for (Map.Entry<String, String> context : ((Map<String, String>) mbscnx.getAttribute(versionedEjbOrWar.getValue(),
                    "Contexts")).entrySet()) {
                ObjectName j2eeApplication;
                String applicationName = versionedEjbOrWar.getKey().getKeyProperty("J2EEApplication");
                if (applicationName == null || applicationName.equals("none") || applicationName.equals("null")) {
                    // Not an EAR, therefore applicationName is the WAR or
                    // EJB-JAR archive's name
                    applicationName = versionedEjbOrWar.getKey().getKeyProperty("name");
                    j2eeApplication = versionedEjbOrWar.getKey();

                    // Filter out bizarre characters from applicationName
                    int doubleSlash = applicationName.indexOf("//");
                    if (doubleSlash >= 0) {
                        int nextSlash = applicationName.indexOf('/', doubleSlash + 2);
                        if (nextSlash >= 0) {
                            // URL like //localhost/applicationName
                            applicationName = applicationName.substring(nextSlash + 1);
                        }
                    }

                    if (applicationName.length() == 0) {
                        applicationName = "[ ROOT ]";
                    }
                } else {
                    Set<ObjectName> j2eeApplications = mbscnx.queryNames(new ObjectName("*:j2eeType=J2EEApplication,name="
                            + applicationName + ",*"), null);
                    if (j2eeApplications.size() == 0) {
                        // Application not found... Deployment issue?
                        continue;
                    }
                    j2eeApplication = j2eeApplications.iterator().next();
                }

                // Get version identifier from the context
                String version;
                int versionIDStart = context.getKey().lastIndexOf("version");
                if (versionIDStart > 0) {
                    versionIDStart += "version".length();
                    int versionIDEnd = context.getKey().indexOf('/', versionIDStart);
                    if (versionIDEnd > 0) {
                        version = context.getKey().substring(versionIDStart, versionIDEnd);
                    } else {
                        version = context.getKey().substring(versionIDStart);
                    }
                    applicationName = applicationName.replace("-version" + version, "");
                } else {
                    // No version, ignore
                    continue;
                }
                if (applicationName.contains("-version")) {
                    // Not currently processed version, ignore
                    continue;
                }

                ApplicationInformation applicationInformation = applications.get(applicationName);
                if (applicationInformation == null) {
                    applicationInformation = new ApplicationInformation();
                    applications.put(applicationName, applicationInformation);
                }
                applicationInformation.versions.put(version, new VersionInformation(context.getValue(), j2eeApplication));
                applicationInformation.managers.add(versionedEjbOrWar.getValue());
            }
        }

        // Process non-versioned EARs. This will also clean up
        // nonVersionedEjbsAndWars to make it have only the standalone archives
        for (ObjectName ear : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=J2EEApplication,*"), null)) {
            String applicationName = ear.getKeyProperty("name");
            if (!applicationName.contains("-version") && !applications.containsKey(applicationName)) {
                applications.put(applicationName, null);

                for (String module : (String[]) mbscnx.getAttribute(ear, "modules")) {
                    nonVersionedEjbsAndWars.remove(new ObjectName(module));
                }
            }
        }

        // Process non-versioned standalone archives
        for (ObjectName nonVersionedEjbOrWar : nonVersionedEjbsAndWars) {
            String applicationName = nonVersionedEjbOrWar.getKeyProperty("name");

            // Filter out bizarre characters from applicationName
            int doubleSlash = applicationName.indexOf("//");
            if (doubleSlash >= 0) {
                int nextSlash = applicationName.indexOf('/', doubleSlash + 2);
                if (nextSlash >= 0) {
                    // URL like //localhost/applicationName
                    applicationName = applicationName.substring(nextSlash + 1);
                }
            }

            if (applicationName.length() == 0) {
                applicationName = "[ ROOT ]";
            }

            if (!applications.containsKey(applicationName)) {
                applications.put(applicationName, null);
            }
        }

        return applications;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#listOfApplications()
     */
    @SuppressWarnings("unchecked")
    @Override
    public List<ApplicationBean> listOfApplications() throws Exception {
        checkJMXConnection();
        appendToLog("Getting list of applications");
        List<ApplicationBean> applications = new ArrayList<ApplicationBean>();

        appendToLog("   Getting deployed applications");
        for (Map.Entry<String, ApplicationInformation> application : getApplicationsList().entrySet()) {
            if (application.getValue() == null) {
                ApplicationBean applicationBean = new ApplicationBean(application.getKey());
                applicationBean.setState(ServerAction.STATE_DEPLOYED);
                applications.add(applicationBean);
            } else {
                for (Map.Entry<String, VersionInformation> versionInformation : application.getValue().versions.entrySet()) {
                    ApplicationBean applicationBean = new ApplicationBean(application.getKey(), versionInformation.getKey());
                    applicationBean.setState(ServerAction.STATE_DEPLOYED);
                    applicationBean.setPolicy(versionInformation.getValue().policy);
                    applications.add(applicationBean);
                }
            }
        }

        appendToLog("   Getting present yet not deployed applications");
        for (String application : (List<String>) mbscnx.getAttribute(j2eeServer, "deployableEars")) {
            ApplicationBean applicationBean = new ApplicationBean(new File(application).getName());
            applicationBean.setState(ServerAction.STATE_PRESENT);
            applications.add(applicationBean);
        }

        for (String application : (List<String>) mbscnx.getAttribute(j2eeServer, "deployableJars")) {
            ApplicationBean applicationBean = new ApplicationBean(new File(application).getName());
            applicationBean.setState(ServerAction.STATE_PRESENT);
            applications.add(applicationBean);
        }

        for (String application : (List<String>) mbscnx.getAttribute(j2eeServer, "deployableWars")) {
            ApplicationBean applicationBean = new ApplicationBean(new File(application).getName());
            applicationBean.setState(ServerAction.STATE_PRESENT);
            applications.add(applicationBean);
        }

        appendToLog("Got the list of applications");
        return applications;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#upload(org.ow2.jasmine.jadort.service.entities.ApplicationBean)
     */
    @Override
    public String upload(final ApplicationBean application) throws Exception {
        checkJMXConnection();
        appendToLog("Uploading ApplicationBean '" + application.toString() + "'");

        FileInputStream inputStream = new FileInputStream(application.getFile());
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] buf = new byte[JonasServerAction.BUFFER_SIZE];
        // Read bytes
        int len;
        while ((len = inputStream.read(buf)) > 0) {
            baos.write(buf, 0, len);
        }
        byte[] bytesOfFile = baos.toByteArray();

        Object[] opParams = new Object[] { bytesOfFile, application.getFile().getName(), false };
        String[] opSignature = new String[] { "[B", "java.lang.String", "boolean" };

        String sendFile = (String) mbscnx.invoke(j2eeServer, "sendFile", opParams, opSignature);
        // Manually create the pseudo-URL to avoid exceptions if the sending
        // server and the sent server don't use the same OS (UNIX doesn't like :
        // in a filename).
        URL url = new URL("file:" + sendFile);
        appendToLog("Call to J2EEServer.sendFile successful, ApplicationBean saved as '" + url.toString() + "'");
        return url.getFile();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#deploy(java.lang.String)
     */
    @Override
    public void deploy(final String appName) throws Exception {
        checkJMXConnection();
        appendToLog("Deploying application '" + appName + "'");

        Object[] opParams = new Object[] { appName };
        String[] opSignature = new String[] { "java.lang.String" };
        mbscnx.invoke(j2eeServer, "deploy", opParams, opSignature);
        appendToLog("Call to J2EEServer.deploy successful for application '" + appName + "'");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#setDefault(java.lang.String)
     */
    @SuppressWarnings("unchecked")
    @Override
    public String setDefault(final String appName) throws Exception {
        checkJMXConnection();
        appendToLog("Setting application '" + appName + "' as default");

        String applicationName = null;
        // Find the application
        {
            URL url = new URL("file:" + appName);

            if (applicationName == null) {
                for (ObjectName ear : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=J2EEApplication,*"), null)) {
                    if (url.equals(mbscnx.getAttribute(ear, "earUrl"))) {
                        applicationName = ear.getKeyProperty("name");
                    }
                }
            }

            if (applicationName == null) {
                for (ObjectName ejbjar : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=EJBModule,*"), null)) {
                    if (url.equals(mbscnx.getAttribute(ejbjar, "url"))) {
                        applicationName = ejbjar.getKeyProperty("name");
                    }
                }
            }

            if (applicationName == null) {
                for (ObjectName war : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=WebModule,*"), null)) {
                    if (url.equals(mbscnx.getAttribute(war, "warURL"))) {
                        applicationName = war.getKeyProperty("name");
                    }
                }
            }
        }

        if (applicationName == null) {
            throw new IllegalArgumentException("No application for appName '" + appName + "'");
        }

        // Get version identifier
        String version = null;
        int versionIDStart = applicationName.lastIndexOf("version");
        if (versionIDStart > 0) {
            versionIDStart += "version".length();
            int versionIDEnd = applicationName.indexOf('/', versionIDStart);
            if (versionIDEnd > 0) {
                version = applicationName.substring(versionIDStart, versionIDEnd);
            } else {
                version = applicationName.substring(versionIDStart);
            }
            applicationName = applicationName.replace("-version" + version, "");
        }

        if (version == null) {
            throw new IllegalArgumentException("Application '" + applicationName + "' is not versioned");
        }

        String filteredApplicationName = applicationName;
        // Filter out bizarre characters from applicationName
        int doubleSlash = filteredApplicationName.indexOf("//");
        if (doubleSlash >= 0) {
            int nextSlash = filteredApplicationName.indexOf('/', doubleSlash + 2);
            if (nextSlash >= 0) {
                // URL like //localhost/applicationName
                filteredApplicationName = filteredApplicationName.substring(nextSlash + 1);
            }
        }

        // Get versions and managers
        ApplicationInformation applicationInformation = getApplicationsList().get(filteredApplicationName);

        if (applicationInformation == null) {
            throw new IllegalArgumentException("Application '" + filteredApplicationName + "' is not in the applications list");
        }
        if (!applicationInformation.versions.containsKey(version)) {
            throw new IllegalArgumentException("Application '" + filteredApplicationName + "' doesn't have version '" + version
                    + "'");
        }

        // Find the old default
        String oldDefaultVersion = null;
        for (Map.Entry<String, VersionInformation> versionInformation : applicationInformation.versions.entrySet()) {
            if (versionInformation.getValue().policy.equals("Default")) {
                if (!versionInformation.getKey().equals(version)) {
                    oldDefaultVersion = "-version" + versionInformation.getKey();
                    setPolicy(applicationInformation.managers, versionInformation.getKey(), "Disabled");
                }
                break;
            }
        }

        // Set new default
        setPolicy(applicationInformation.managers, version, "Default");

        if (oldDefaultVersion != null) {
            // Return old default
            for (ObjectName ear : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=J2EEApplication,*"), null)) {
                String name = ear.getKeyProperty("name");
                if (name.startsWith(applicationName) && name.contains(oldDefaultVersion)) {
                    return ((URL) mbscnx.getAttribute(ear, "earUrl")).getFile();
                }
            }

            for (ObjectName ejbjar : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=EJBModule,*"), null)) {
                String name = ejbjar.getKeyProperty("name");
                if (name.startsWith(applicationName) && name.contains(oldDefaultVersion)) {
                    return ((URL) mbscnx.getAttribute(ejbjar, "url")).getFile();
                }
            }

            for (ObjectName war : (Set<ObjectName>) mbscnx.queryNames(new ObjectName("*:j2eeType=WebModule,*"), null)) {
                String name = war.getKeyProperty("name");
                if (name.startsWith(applicationName) && name.contains(oldDefaultVersion)) {
                    return ((URL) mbscnx.getAttribute(war, "warURL")).getFile();
                }
            }

            throw new FileNotFoundException("Cannot find the file corresponding to application and version '" + applicationName
                    + oldDefaultVersion + "'");
        } else {
            return null;
        }
    }

    @SuppressWarnings("unchecked")
    protected void setPolicy(final Set<ObjectName> managers, final String version, final String policy)
            throws AttributeNotFoundException, InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        appendToLog("   Setting version '" + version + "' as " + policy);
        String versionID_WAR = "-version" + version;
        String versionID_EJB = "version" + version + "/";
        for (ObjectName manager : managers) {
            for (String context : ((Map<String, String>) mbscnx.getAttribute(manager, "Contexts")).keySet()) {
                if (context.contains(versionID_WAR) || context.contains(versionID_EJB)) {
                    Object[] opParams = new Object[] { context, policy };
                    String[] opSignature = new String[] { "java.lang.String", "java.lang.String" };
                    mbscnx.invoke(manager, "rebindContext", opParams, opSignature);
                    break;
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#undeploy(java.lang.String)
     */
    @Override
    public void undeploy(final String appName) throws Exception {
        checkJMXConnection();
        appendToLog("Undeploying application '" + appName + "'");

        Object[] opParams = new Object[] { appName };
        String[] opSignature = new String[] { "java.lang.String" };
        mbscnx.invoke(j2eeServer, "undeploy", opParams, opSignature);
        appendToLog("Call to J2EEServer.undeploy successful for application '" + appName + "'");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#erase(java.lang.String)
     */
    @Override
    public void erase(final String appName) throws Exception {
        checkJMXConnection();
        appendToLog("Erasing application '" + appName + "'");

        Object[] opParams = new Object[] { appName };
        String[] opSignature = new String[] { "java.lang.String" };
        mbscnx.invoke(j2eeServer, "removeModuleFile", opParams, opSignature);
        appendToLog("Call to J2EEServer.removeModuleFile successful for application '" + appName + "'");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.action.ServerAction#getApplicationBean(java.lang.String)
     */
    @SuppressWarnings("unchecked")
    @Override
    public ApplicationBean getApplicationBean(final String appName) throws Exception {
        appendToLog("Getting ApplicationBean for application '" + appName + "'");

        URL url = new URL("file:" + appName);
        for (Map.Entry<String, ApplicationInformation> application : getApplicationsList().entrySet()) {
            if (application.getValue() == null) {
                // Non-versioned application
                ApplicationBean applicationBean = new ApplicationBean(appName);
                applicationBean.setState(ServerAction.STATE_PRESENT);
                return applicationBean;
            } else {
                for (Map.Entry<String, VersionInformation> version : application.getValue().versions.entrySet()) {
                    boolean found = false;
                    try {
                        if (url.equals(mbscnx.getAttribute(version.getValue().mbean, "earUrl"))) {
                            found = true;
                        }
                    } catch (AttributeNotFoundException ignored) {
                        // Ignored
                    }

                    try {
                        if (url.equals(mbscnx.getAttribute(version.getValue().mbean, "url"))) {
                            found = true;
                        }
                    } catch (AttributeNotFoundException ignored) {
                        // Ignored
                    }

                    try {
                        if (url.equals(mbscnx.getAttribute(version.getValue().mbean, "warURL"))) {
                            found = true;
                        }
                    } catch (AttributeNotFoundException ignored) {
                        // Ignored
                    }

                    if (found) {
                        ApplicationBean applicationBean = new ApplicationBean();
                        applicationBean.setName(application.getKey());
                        applicationBean.setPolicy(version.getValue().policy);
                        applicationBean.setState(ServerAction.STATE_DEPLOYED);
                        applicationBean.setVersion(version.getKey());
                        appendToLog("ApplicationBean for application '" + appName + "' is a deployed application, returning '"
                                + applicationBean.toString() + "'");
                        return applicationBean;
                    }
                }
            }
        }

        for (String application : (List<String>) mbscnx.getAttribute(j2eeServer, "deployableEars")) {
            if (url.equals(new URL("file:" + application))) {
                ApplicationBean applicationBean = new ApplicationBean(new File(application).getName());
                applicationBean.setState(ServerAction.STATE_PRESENT);
                appendToLog("ApplicationBean for application '" + appName + "' is a present application, returning '"
                        + applicationBean.toString() + "'");
                return applicationBean;
            }
        }

        for (String application : (List<String>) mbscnx.getAttribute(j2eeServer, "deployableJars")) {
            if (url.equals(new URL("file:" + application))) {
                ApplicationBean applicationBean = new ApplicationBean(new File(application).getName());
                applicationBean.setState(ServerAction.STATE_PRESENT);
                appendToLog("ApplicationBean for application '" + appName + "' is a present application, returning '"
                        + applicationBean.toString() + "'");
                return applicationBean;
            }
        }

        for (String application : (List<String>) mbscnx.getAttribute(j2eeServer, "deployableWars")) {
            if (url.equals(new URL("file:" + application))) {
                ApplicationBean applicationBean = new ApplicationBean(new File(application).getName());
                applicationBean.setState(ServerAction.STATE_PRESENT);
                appendToLog("ApplicationBean for application '" + appName + "' is a present application, returning '"
                        + applicationBean.toString() + "'");
                return applicationBean;
            }
        }

        appendToLog("Application '" + appName + "' not found");
        return null;
    }
}
