/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerAction.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.action;

import java.lang.reflect.Constructor;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;

/**
 * Action for a server (JOnAS or other)
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public abstract class ServerAction extends AbstractAction {

    public static String STATE_PRESENT = "Present";
    public static String STATE_DEPLOYED = "Deployed";

    private static Constructor<? extends ServerAction> constructor;
    private static Map<String, ServerAction> pool = new Hashtable<String, ServerAction>();

    public static ServerAction getServerAction(final ServerBean server) {
        synchronized (ServerAction.pool) {
            String key = ServerAction.getKey(server);
            ServerAction serverAction = ServerAction.pool.get(key);
            if (serverAction == null) {
                serverAction = ServerAction.newInstance(server);
                ServerAction.pool.put(key, serverAction);
            }
            return serverAction;
        }
    }

    private static String getKey(final ServerBean server) {
        StringBuilder sb = new StringBuilder();
        sb.append(server.getName());
        sb.append(server.getConnector().getConnectorUrl());
        if (server.getConnector().getUsername() != null && server.getConnector().getPassword() != null) {
            sb.append(server.getConnector().getUsername());
            sb.append(server.getConnector().getPassword());
        }
        return sb.toString();
    }

    protected static ServerAction newInstance(final ServerBean server) {
        try {
            if (ServerAction.constructor == null) {
                ServerAction.getConstructor(JonasServerAction.class);
            }
            return ServerAction.constructor.newInstance(server);
        } catch (Exception e) {
            throw new IllegalArgumentException("Failed creating the server action", e);
        }
    }

    @SuppressWarnings("unchecked")
    protected static void getConstructor(final Class<? extends ServerAction> actionType) throws Exception {
        Constructor[] constructors = actionType.getDeclaredConstructors();
        for (Constructor<? extends ServerAction> constructor : constructors) {
            Class<?>[] params = constructor.getParameterTypes();
            if (params.length == 1 && params[0] == ServerBean.class) {
                ServerAction.constructor = constructor;
            }
        }
    }

    /**
     * @param actionType The action type to set, that'll be used in the
     *        {@link ServerAction#getServerAction(ServerBean)} method.
     */
    public static void setActionType(final Class<? extends ServerAction> actionType) {
        try {
            ServerAction.getConstructor(actionType);
        } catch (Exception e) {
            throw new IllegalArgumentException("Failed setting ActionType", e);
        }
    }

    /**
     * @return List of applications currently present on this server,
     *         reformatted as ApplicationBean objects.
     */
    public abstract List<ApplicationBean> listOfApplications() throws Exception;

    /**
     * @param application ApplicationBean to upload.
     * @return Name of the application as saved by the server, to use in the
     *         {@link ServerAction#deploy(String)} and
     *         {@link ServerAction#setDefault(String)},
     *         {@link ServerAction#undeploy(String)} and
     *         {@link ServerAction#erase(String)} methods.
     */
    public abstract String upload(final ApplicationBean application) throws Exception;

    /**
     * @param appName Name of the application, as returned by
     *        {@link ServerAction#upload(ApplicationBean)} or
     *        {@link ServerAction#setDefault(String)}, for which to obtain an
     *        {@link ApplicationBean}.
     * @return {@link ApplicationBean} corresponding to appName, null if nothing
     *         matches.
     */
    public abstract ApplicationBean getApplicationBean(final String appName) throws Exception;

    /**
     * @param appName ApplicationBean to deploy. You must use the string
     *        returned by {@link ServerAction#upload(ApplicationBean)} or
     *        {@link ServerAction#setDefault(String)}.
     */
    public abstract void deploy(final String appName) throws Exception;

    /**
     * @param appName ApplicationBean to set as default version (previous
     *        Default version will become Disabled). You must use the string
     *        returned by {@link ServerAction#upload(ApplicationBean)} or
     *        {@link ServerAction#setDefault(String)}.
     * @return Old application that was default.
     */
    public abstract String setDefault(final String appName) throws Exception;

    /**
     * @param appName ApplicationBean to undeploy. You must use the string
     *        returned by {@link ServerAction#upload(ApplicationBean)} or
     *        {@link ServerAction#setDefault(String)}.
     */
    public abstract void undeploy(final String appName) throws Exception;

    /**
     * @param appName ApplicationBean to erase. You must use the string returned
     *        by {@link ServerAction#upload(ApplicationBean)} or
     *        {@link ServerAction#setDefault(String)}.
     */
    public abstract void erase(final String appName) throws Exception;

}
