/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DeploymentBean.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.entities;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToOne;
import javax.persistence.Version;

/**
 * Bean that represents a deployment (progress of a server migration)
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
@Entity
public class DeploymentBean implements Serializable {

    /**
     * For serialization
     */
    private static final long serialVersionUID = -2851071984012556925L;

    /**
     * Id of the deployment, use for persistence. Auto generated.
     */
    @Id
    @GeneratedValue
    private Integer id;

    /**
     * Verison of the deployment. Use for persistence. Auto generated.
     */
    @Version
    @SuppressWarnings("unused")
    private Integer version;

    /**
     * Different possible progress states of a deployment.
     */
    public enum DeploymentState {
        INITIAL, UPLOAD_OK, DEPLOY_OK, SET_DEFAULT_OK, UNDEPLOY_OK, ERASE_OK
    };

    /**
     * Different states of a deployment.
     */
    public enum ActionState {
        WAITING, RUNNING, FINISHED_OK, FINISHED_ERROR
    };

    /**
     * The associated server to the deployment.
     */
    // Cascade for all except CascadeType.REMOVE, since removing a
    // DeploymentBean doesn't imply that the associated ServerBean (created as
    // part of a GroupBean) should be deleted!
    @OneToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    private ServerBean server;

    /**
     * Has the server been selected?
     */
    private boolean selected;

    /**
     * Old application name. This is the previously deployed applicatin on the
     * server.
     */
    private String oldApplication;

    /**
     * New application name. The application the deployment is currently
     * deploying.
     */
    private String newApplication;

    /**
     * Progress percent of the deployment.
     */
    private Integer progress = 0;

    /**
     * Deployment log
     */
    private String log;

    /**
     * Deployment progress state
     */
    @Enumerated(EnumType.STRING)
    private DeploymentState deploymentState;

    /**
     * Deployment state
     */
    @Enumerated(EnumType.STRING)
    private ActionState actionState;

    /**
     * Default deployment constructor
     */
    public DeploymentBean() {
        this.selected = false;
        this.deploymentState = DeploymentState.INITIAL;
        this.actionState = ActionState.WAITING;
    }

    /**
     * Construct a deployment with its associated server
     *
     * @param server The server associated ti the depoyment
     */
    public DeploymentBean(final ServerBean server) {
        this();
        this.server = server;
    }

    /**
     * Server getter
     *
     * @return The associated server
     */
    public ServerBean getServer() {
        return this.server;
    }

    /**
     * Associated server name getter
     *
     * @return The associated server name
     */
    public String getServerName() {
        return this.server.getName();
    }

    /**
     * Do not use this method, needs to be here to satisfy action script
     *
     * @param ServerName
     */
    public void setServerName(final String ServerName) {
        // Empty method, needs to be here to satisfy AS
    }

    /**
     * Change the associated server
     *
     * @param server New associated server
     */
    public void setServer(final ServerBean server) {
        this.server = server;
    }

    /**
     * "Has the server been selected?" getter
     *
     * @return Whether the server has been selected
     */
    public boolean isSelected() {
        return this.selected;
    }

    /**
     * "Has the server been selected?" setter
     *
     * @param selected Whether the server has been selected
     */
    public void setSelected(final boolean selected) {
        this.selected = selected;
    }

    /**
     * Return the deployment state in String format : <br /> - WAITING -->
     * "Waiting" <br /> - RUNNING --> "Running" <br /> - FINISHED_OK -- > "Done
     * OK" <br /> - FINISHED_ERROR --> "Error !" <br />
     *
     * @return
     */
    public String getState() {
        // This method is used by AS, all changes should be reflected to the
        // Flex interface as well
        if (this.actionState == ActionState.WAITING) {
            return "Waiting";
        } else if (this.actionState == ActionState.RUNNING) {
            return "Running";
        } else if (this.actionState == ActionState.FINISHED_OK) {
            return "Done OK";
        } else if (this.actionState == ActionState.FINISHED_ERROR) {
            return "Error !";
        } else {
            return null;
        }
    }

    /**
     * Do not use this method, needs to be here to satisfy action script
     *
     * @param state
     */
    public void setState(final String state) {
        // Empty method, needs to be here to satisfy AS
    }

    /**
     * Old application name getter
     *
     * @return Old application name
     */
    public String getOldApplication() {
        return this.oldApplication;
    }

    /**
     * Old application name setter.
     *
     * @param oldApplication Old application name
     */
    public void setOldApplication(final String oldApplication) {
        this.oldApplication = oldApplication;
    }

    /**
     * @return true if there's an old application version present and the old
     *         application is not the same as the new one.
     */
    public boolean hasOldVersion() {
        return oldApplication != null && !oldApplication.equals(newApplication);
    }

    /**
     * New application name getter.
     *
     * @return New application name
     */
    public String getNewApplication() {
        return this.newApplication;
    }

    /**
     * New application name setter.
     *
     * @param newApplication New application name
     */
    public void setNewApplication(final String newApplication) {
        this.newApplication = newApplication;
    }

    /**
     * Percent of progress getter
     *
     * @return The deployment progress (percents)
     */
    public Integer getProgress() {
        return this.progress;
    }

    /**
     * Deployment progress setter.
     *
     * @param progress The progress percent to set.
     */
    public void setProgress(final Integer progress) {
        this.progress = progress;
    }

    /**
     * Get the deployment log
     *
     * @return Deployment log
     */
    public String getLog() {
        return this.log;
    }

    /**
     * Set the deployment log
     *
     * @param log The log to set for this deployment
     */
    public void setLog(final String log) {
        this.log = log;
    }

    /**
     * Add log to the deployment log.
     *
     * @param string The log to add for this deployment
     */
    public void appendToLog(final String string) {
        if (string != null && string.length() > 0) {
            if (log == null) {
                log = string;
            } else {
                log += '\n' + string;
            }
        }
    }

    /**
     * Get the deployment progress state
     *
     * @return Deployment progress state
     */
    public DeploymentState getDeploymentState() {
        return this.deploymentState;
    }

    /**
     * Set the deployment progress state
     *
     * @param deploymentState the deployment progress state to set
     */
    public void setDeploymentState(final DeploymentState deploymentState) {
        this.deploymentState = deploymentState;
    }

    /**
     * Get deployment state.
     *
     * @return Deployment state.
     */
    public ActionState getActionState() {
        return this.actionState;
    }

    /**
     * Set deployment state.
     *
     * @param actionState Deployment state to set.
     */
    public void setActionState(final ActionState actionState) {
        this.actionState = actionState;
    }

    /**
     * Returns a string that displays and identifies this object's properties.
     *
     * @return A string that displays and identifies this object's properties.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.getClass().getSimpleName());
        sb.append("[server='");
        sb.append(server);
        sb.append("']");
        return sb.toString();
    }

    /**
     * Get the deployment id
     *
     * @return Deployment id
     */
    public Integer getId() {
        return this.id;
    }

}
