/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: OperationStateBean.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.entities;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Version;

/**
 * Bean that represents an operation
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
@Entity
public class OperationStateBean implements Serializable {

    /**
     * Used for serialization
     */
    private static final long serialVersionUID = -4060088928914512618L;

    /**
     * Define the different steps of an operation
     */
    public enum Step {
        SELECT_OPERATION, INITIALIZE_TOPOLOGY, SELECT_GROUP, SELECT_APPLICATION, EXECUTING_MIGRATION_PART1, EXECUTING_MIGRATION_PART2, FINISHED
    };

    /**
     * Operation id, use for persistence
     */
    @Id
    @GeneratedValue
    @SuppressWarnings("unused")
    private Integer id;

    /**
     * Verison of the deployment. Use for persistence. Auto generated.
     */
    @Version
    @SuppressWarnings("unused")
    private Integer version;

    /**
     * Operation creation date
     */
    @Temporal(TemporalType.TIMESTAMP)
    private Date date;

    /**
     * Operation name
     */
    private String name;

    /**
     * The topology to use with this operation
     */
    @OneToOne(cascade = CascadeType.ALL)
    private TopologyBean topology;

    /**
     * Active deployments of the operation
     */
    @OneToMany(cascade = CascadeType.ALL)
    private List<DeploymentBean> deploymentlist;

    /**
     * The application the operation has to deploy
     */
    @OneToOne(cascade = CascadeType.ALL)
    private ApplicationBean application;

    /**
     * The state of deployment the operation has to reach (use for the
     * deployment phase (upload/deploy/set default/...)
     */
    @Enumerated(EnumType.STRING)
    private DeploymentBean.DeploymentState aimedDeploymentState = DeploymentBean.DeploymentState.INITIAL;

    /**
     * The progress of deployment the operation has to reach (percent)
     */
    @Enumerated(EnumType.STRING)
    private Integer aimedDeploymentProgress = 0;

    /**
     * Current operation step
     */
    @Enumerated(EnumType.STRING)
    private Step currentStep = Step.SELECT_OPERATION;

    /**
     * The selected group of server
     */
    @OneToOne
    private GroupBean selectedGroup;

    /**
     * Get the deployment step to reach
     *
     * @return deployment step to reach
     */
    public DeploymentBean.DeploymentState getAimedDeploymentState() {
        return this.aimedDeploymentState;
    }

    /**
     * Set the deployment step to reach
     *
     * @param globalDeploymentState the new deployment step to reach
     */
    public void setAimedDeploymentState(final DeploymentBean.DeploymentState globalDeploymentState) {
        this.aimedDeploymentState = globalDeploymentState;
    }

    /**
     * Get the deployment progress to reach
     *
     * @return deployment progress to reach
     */
    public Integer getAimedDeploymentProgress() {
        return this.aimedDeploymentProgress;
    }

    /**
     * Set the deployment progress to reach
     *
     * @param globalDeploymentProgress the new deployment progress to reach
     */
    public void setAimedDeploymentProgress(final Integer globalDeploymentProgress) {
        this.aimedDeploymentProgress = globalDeploymentProgress;
    }

    /**
     * Get the current operation step
     *
     * @return the current operation step
     */
    public Step getCurrentStep() {
        return this.currentStep;
    }

    /**
     * Set the current operation step
     *
     * @param step the current operation step
     */
    public void setCurrentStep(final Step step) {
        this.currentStep = step;
    }

    /**
     * Get the selected group
     *
     * @return the selected group
     */
    public GroupBean getSelectedGroup() {

        // Return a copy of the selected group as GroupBean contains list
        // with lazy reload

        return this.selectedGroup;
    }

    /**
     * Set the selected group
     *
     * @param selectedGroup The group to select
     */
    public void setSelectedGroup(final GroupBean selectedGroup) {
        this.selectedGroup = selectedGroup;
    }

    /**
     * Get the topology
     *
     * @return the topology
     */
    public TopologyBean getTopology() {
        return this.topology;
    }

    /**
     * Set the topology
     *
     * @param topology The topology to set
     */
    public void setTopology(final TopologyBean topology) {
        this.topology = topology;
    }

    /**
     * Get the list of deployments that compose the operation
     *
     * @return the list of deployments that compose the operation
     */
    public List<DeploymentBean> getDeploymentlist() {
        return this.deploymentlist;
    }

    /**
     * Set the list of deployments that compose the operation
     *
     * @param deploymentlist the new list of deployments that compose the
     *        operation
     */
    public void setDeploymentlist(final List<DeploymentBean> deploymentlist) {
        this.deploymentlist = deploymentlist;
    }

    /**
     * Get the application to ddeploy
     *
     * @return the application to deploy
     */
    public ApplicationBean getApplication() {
        return this.application;
    }

    /**
     * Set the application to deploy
     *
     * @param application the application to deploy
     */
    public void setApplication(final ApplicationBean application) {
        this.application = application;
    }

    /**
     * Get the operation creation date
     *
     * @return the operation creation date
     */
    public Date getDate() {
        return date;
    }

    /**
     * Set the operation creation date
     *
     * @param date the operation creation date
     */
    public void setDate(final Date date) {
        this.date = date;
    }

    /**
     * Get the operation name
     *
     * @return the operation name
     */
    public String getName() {
        return this.name;
    }

    /**
     * Set the operation name
     *
     * @param name the operation name
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * Returns a string that displays and identifies this object's properties.
     *
     * @return A string that displays and identifies this object's properties.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.getClass().getSimpleName());
        sb.append("[aimedDeploymentState='");
        sb.append(aimedDeploymentState);
        sb.append("', currentStep='");
        sb.append(currentStep);
        sb.append("']");
        return sb.toString();
    }

    /**
     * Get the operation id
     *
     * @return the operation id
     */
    public Integer getId() {
        return this.id;
    }

    /**
     * Set the operation id
     */
    public void setId(final Integer id) {
        this.id = id;
    }

}
