/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JadortServiceImpl.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.implementation;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateful;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.InitialContext;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.ow2.jasmine.jadort.service.IJadortService;
import org.ow2.jasmine.jadort.service.JadortServiceException;
import org.ow2.jasmine.jadort.service.action.ServerAction;
import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean;
import org.ow2.jasmine.jadort.service.entities.GroupBean;
import org.ow2.jasmine.jadort.service.entities.JMXConnectorBean;
import org.ow2.jasmine.jadort.service.entities.ModJKBean;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;
import org.ow2.jasmine.jadort.service.entities.TopologyBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean.ActionState;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean.DeploymentState;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean.Step;
import org.ow2.jasmine.jadort.service.topology.xml.JMXConnector;
import org.ow2.jasmine.jadort.service.topology.xml.ModJK;
import org.ow2.jasmine.jadort.service.topology.xml.Server;
import org.ow2.jasmine.jadort.service.topology.xml.Topology;

/**
 * This class provides default implementations for the IJadortService interface.
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
@Local(value = IJadortService.class)
@Remote(value = IJadortService.class)
@Stateful(mappedName = "JadortConsoleStatefulBean")
public class JadortServiceImpl implements IJadortService {

    final static int BUFFER_SIZE = 16384;

    // These constants are shared within the package
    final static String QUEUE_NAME = "queue/JadortServiceJMSQueue";
    final static String DEPLOYMENT_ID = "deploymentId";
    final static String APPLICATION_ID = "applicationId";
    final static String ACTION_TYPE = "actionType";
    final static String APPLICATION_NAME = "applicationName";

    protected final static String XSD_PATH = "XMLSchema/topology.xsd";

    /**
     * Action type<br>
     * <br>
     * <b>UPLOAD :</b> upload a new version <br>
     * <b>DEPLOY :</b> deploy the new version <br>
     * <b>SET_DEFAULT:</b> make new version as default, activate new version
     * and deactivate old version <br>
     * <b>UNDEPLOY:</b> undeploy old version <br>
     * <b>ERASE :</b> erase old version
     */
    protected enum ActionType {
        UPLOAD, DEPLOY, SET_DEFAULT, UNDEPLOY, ERASE
    }

    protected Unmarshaller unMarshaller;

    protected OperationStateBean operation;

    @PersistenceContext
    protected EntityManager em;

    private QueueConnection qc;
    private QueueSession session;
    private Queue queue;

    @PostConstruct
    protected void initialize() throws JadortServiceException {
        try {
            InitialContext ctx = new InitialContext();
            queue = (Queue) ctx.lookup(JadortServiceImpl.QUEUE_NAME);
            QueueConnectionFactory factory = (QueueConnectionFactory) ctx.lookup("QCF");
            qc = factory.createQueueConnection();
            session = qc.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
        } catch (Exception e) {
            JadortServiceException exception = new JadortServiceException("Failed initializing a JadortService stateful bean: "
                    + e.getMessage(), e);
            System.out.println(exception.getMessage());
            e.printStackTrace();
            throw exception;
        }
    }

    @PreDestroy
    protected void terminate() throws JadortServiceException {
        try {
            session.close();
            qc.close();
        } catch (Exception e) {
            JadortServiceException exception = new JadortServiceException("Failed terminating a JadortService stateful bean: "
                    + e.getMessage(), e);
            System.out.println(exception.getMessage());
            e.printStackTrace();
            throw exception;
        }
    }

    /**
     * Returns the list of operations
     *
     * @return List of Operations
     * @see org.ow2.jasmine.jadort.service.IJadortService#getOperationsList()
     * @throws JadortServiceException if the getOperationsList() method can not
     *         be called in the current step
     */
    @SuppressWarnings("unchecked")
    public List<OperationStateBean> getOperationsList() throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.SELECT_OPERATION);

            List<OperationStateBean> result = new ArrayList<OperationStateBean>();

            // Create a dummy list which contains only the needed fields
            for (OperationStateBean operation : (List<OperationStateBean>) em.createQuery(
                    "select o from OperationStateBean o order by o.date asc").getResultList()) {
                OperationStateBean temp = new OperationStateBean();
                temp.setAimedDeploymentState(operation.getAimedDeploymentState());
                temp.setAimedDeploymentProgress(operation.getAimedDeploymentProgress());
                temp.setCurrentStep(operation.getCurrentStep());
                temp.setDate(operation.getDate());
                temp.setId(operation.getId());
                temp.setName(operation.getName());
                result.add(temp);
            }
            return result;
        }
    }

    /**
     * Returns the current operation
     *
     * @return The current operation.
     * @see org.ow2.jasmine.jadort.service.IJadortService#getCurrentOperation()
     * @throws JadortServiceException
     */
    public OperationStateBean getCurrentOperation() throws JadortServiceException {
        synchronized (this) {
            refreshOperation();

            if (operation == null) {
                return null;
            } else {
                // Return a copy since operation has been loaded by lazy
                // initialization. Remember to copy all List attributes as well!
                OperationStateBean bean = new OperationStateBean();
                bean.setAimedDeploymentState(operation.getAimedDeploymentState());
                bean.setAimedDeploymentProgress(operation.getAimedDeploymentProgress());
                bean.setApplication(operation.getApplication());
                bean.setCurrentStep(operation.getCurrentStep());
                bean.setDate(operation.getDate());
                bean.setDeploymentlist(fetchDeployments());
                bean.setId(operation.getId());
                bean.setName(operation.getName());
                bean.setSelectedGroup(fetchSelectedGroup());
                if (operation.getTopology() != null) {
                    TopologyBean topology = new TopologyBean();
                    topology.setGroups(fetchGroups());
                    topology.setId(operation.getTopology().getId());
                    bean.setTopology(topology);
                }
                return bean;
            }
        }
    }

    /**
     * Loads the topology from a .xml file.
     *
     * @param xmlTopoFile - the file path of the .xml file uploaded by the user,
     *        this file contains the topology description
     * @see org.ow2.jasmine.jadort.service.IJadortService#loadTopology(java.io.File)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the loadTopology method is called outside the
     *         INITIALIZE_TOPOLOGY Step,
     *         <li>if an error is occurred when parsing the .xml file
     *         <li>if a problem is occurred when getting the applications on
     *         the servers
     *         </ul>
     */
    public void loadTopology(final File xmlTopoFile) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.INITIALIZE_TOPOLOGY);

            // If JAXB hasn't been initialized yet, initialize it
            if (unMarshaller == null) {
                try {
                    JAXBContext jc = JAXBContext.newInstance(org.ow2.jasmine.jadort.service.topology.xml.Topology.class
                            .getPackage().getName());

                    unMarshaller = jc.createUnmarshaller();

                    SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

                    Schema schema = null;

                    URL url = TopologyBean.class.getClassLoader().getResource(JadortServiceImpl.XSD_PATH);
                    schema = schemaFactory.newSchema(url);

                    unMarshaller.setSchema(schema);
                } catch (Exception e) {
                    unMarshaller = null;
                    throw new JadortServiceException("Exception creating the topology XML unMarshaller: " + e.getMessage(), e);
                }
            }

            Topology xmlTopology;
            try {
                xmlTopology = (Topology) unMarshaller.unmarshal(xmlTopoFile);
            } catch (UnmarshalException e) {
                throw new JadortServiceException(xmlTopoFile.getName() + " is not a valid topology file (checked against "
                        + JadortServiceImpl.XSD_PATH + ")", e);
            } catch (JAXBException e) {
                throw new JadortServiceException("Error parsing the XML topology file: " + e.getMessage(), e);
            }

            TopologyBean topologyBean = new TopologyBean();
            operation.setTopology(topologyBean);

            List<GroupBean> groups = new ArrayList<GroupBean>();
            for (Topology.Group groupXML : xmlTopology.getGroup()) {
                GroupBean groupBean = new GroupBean();
                groupBean.setName(groupXML.getName());
                groupBean.setType(groupXML.getType());

                List<ServerBean> servers = new ArrayList<ServerBean>();
                for (Server serverXML : groupXML.getServers().getServer()) {
                    ServerBean serverBean = new ServerBean();
                    serverBean.setName(serverXML.getName());
                    if (serverXML.getCapacity() != null) {
                        serverBean.setCapacity(serverXML.getCapacity());
                    }
                    serverBean.setConnector(createJMXConnectorBean(serverXML.getJMXConnector()));
                    servers.add(serverBean);
                }
                groupBean.setServers(servers);

                List<ModJKBean> modJK = new ArrayList<ModJKBean>();
                if (groupXML.getModJKs() != null) {
                    for (ModJK modJKXML : groupXML.getModJKs().getModJK()) {
                        ModJKBean modJKBean = new ModJKBean();
                        modJKBean.setName(modJKXML.getName());
                        modJKBean.setConnector(createJMXConnectorBean(modJKXML.getJMXConnector()));
                        modJK.add(modJKBean);
                    }
                }
                groupBean.setModJK(modJK);
                groups.add(groupBean);
            }
            topologyBean.setGroups(groups);

            mergeOperation();
        }
    }

    /**
     * @param jmxConnector
     * @return
     */
    protected JMXConnectorBean createJMXConnectorBean(final JMXConnector jmxConnector) {
        synchronized (this) {
            JMXConnectorBean jmxConnectorBean = new JMXConnectorBean();

            jmxConnectorBean.setConnectorUrl(jmxConnector.getURL());
            if (jmxConnector.getUser() != null && jmxConnector.getPassword() != null) {
                jmxConnectorBean.setUsername(jmxConnector.getUser());
                jmxConnectorBean.setPassword(jmxConnector.getPassword());
            }

            return jmxConnectorBean;
        }
    }

    /**
     * Returns the list of groups obtained from the loaded topology in the
     * current operation
     *
     * @return The list of groups
     * @see org.ow2.jasmine.jadort.service.IJadortService#getGroups()
     * @throws JadortServiceException if the getGroups() method can not be
     *         called in the current step
     */
    public List<GroupBean> getGroups() throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.SELECT_GROUP);
            return fetchGroups();
        }
    }

    /**
     * Returns the list of groups obtained from the loaded topology in the
     * current operation.(returns a copy since
     * operation.getTopology().getGroups() does lazy)
     *
     * @return The list of groups
     */
    private List<GroupBean> fetchGroups() {
        if (operation.getTopology() != null && operation.getTopology().getGroups() != null) {
            // Return a copy since operation.getTopology().getGroups() does lazy
            // initialization. Remember to copy all List attributes as well!
            ArrayList<GroupBean> result = new ArrayList<GroupBean>(operation.getTopology().getGroups().size());
            for (GroupBean groupBean : operation.getTopology().getGroups()) {
                GroupBean groupBeanCopy = new GroupBean();
                groupBeanCopy.setApplications(new ArrayList<ApplicationBean>(groupBean.getApplications()));
                groupBeanCopy.setId(groupBean.getId());
                groupBeanCopy.setModJK(new ArrayList<ModJKBean>(groupBean.getModJK()));
                groupBeanCopy.setName(groupBean.getName());
                groupBeanCopy.setServers(new ArrayList<ServerBean>(groupBean.getServers()));
                groupBeanCopy.setType(groupBean.getType());
                result.add(groupBeanCopy);
            }
            return result;
        } else {
            return null;
        }
    }

    /**
     * Select a group from the group list
     *
     * @param selectedGroup - the group to be selected
     * @see org.ow2.jasmine.jadort.service.IJadortService#selectGroup(org.ow2.jasmine.jadort.service.entities.GroupBean)
     * @throws JadortServiceException if the selectedGroup method can not be
     *         called in the current step
     */
    public void selectGroup(final GroupBean selectedGroup) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.SELECT_GROUP);

            for (GroupBean group : operation.getTopology().getGroups()) {
                if (group.getId().equals(selectedGroup.getId())) {
                    // Remember to copy the oldDeploymentlist array to avoid
                    // LazyInitializationException
                    List<DeploymentBean> oldDeploymentlist = new ArrayList<DeploymentBean>(operation.getDeploymentlist());

                    operation.setSelectedGroup(group);

                    List<DeploymentBean> deploymentlist = new LinkedList<DeploymentBean>();
                    for (ServerBean server : group.getServers()) {
                        DeploymentBean deploymentBean = new DeploymentBean(server);
                        deploymentlist.add(deploymentBean);
                    }
                    operation.setDeploymentlist(deploymentlist);

                    mergeOperation();

                    for (DeploymentBean deploymentBean : oldDeploymentlist) {
                        em.remove(deploymentBean);
                    }
                    em.flush();
                    return;
                }
            }
        }
    }

    /**
     * Returns the list of server deployments in the current operation
     *
     * @return The list of deployments
     * @see org.ow2.jasmine.jadort.service.IJadortService#getDeployments()
     * @throws JadortServiceException if the getDeployments() method can not be
     *         called in the current step. <br>
     *         (this method can be called only in EXECUTING_MIGRATION_PART1 and
     *         EXECUTING_MIGRATION_PART2 steps)
     */
    public List<DeploymentBean> getDeployments() throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.EXECUTING_MIGRATION_PART1, Step.EXECUTING_MIGRATION_PART2, Step.FINISHED);
            return fetchDeployments();
        }
    }

    /**
     * Returns the list of server deployments in the current operation(returns a
     * copy since operation.getDeploymentlist() does lazy)
     *
     * @return The list of deployments
     */
    private List<DeploymentBean> fetchDeployments() {
        if (operation.getDeploymentlist() != null) {
            // Return a copy since operation.getDeploymentlist() does lazy
            // initialization
            return new ArrayList<DeploymentBean>(operation.getDeploymentlist());
        } else {
            return null;
        }
    }

    /**
     * Returns the selected Group in the current operation
     *
     * @return the selected group
     * @see org.ow2.jasmine.jadort.service.IJadortService#getSelectedGroup()
     * @throws JadortServiceException if the getSelectedGroup() method can not
     *         be called in the current step. (this method can be called only in
     *         SELECT_APPLICATION step)
     */

    public GroupBean getSelectedGroup() throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.SELECT_APPLICATION);
            return fetchSelectedGroup();
        }
    }

    /**
     * Returns a copy of the selected Group in the current operation (Return a
     * copy since GroupBean has been initialized lazily)
     *
     * @return the selected group
     */
    private GroupBean fetchSelectedGroup() {
        if (operation.getSelectedGroup() != null) {
            // Return a copy since GroupBean has been initialized lazily
            GroupBean bean = new GroupBean();
            bean.setApplications(new ArrayList<ApplicationBean>(operation.getSelectedGroup().getApplications()));
            bean.setId(operation.getSelectedGroup().getId());
            bean.setModJK(new ArrayList<ModJKBean>(operation.getSelectedGroup().getModJK()));
            bean.setName(operation.getSelectedGroup().getName());
            bean.setServers(new ArrayList<ServerBean>(operation.getSelectedGroup().getServers()));
            bean.setType(operation.getSelectedGroup().getType());
            return bean;
        } else {
            return null;
        }
    }

    /**
     * Returns the deployment object of an server
     *
     * @param server - the server that the method will return his deployment
     * @return the deployment of the server
     */

    private DeploymentBean getDeployment(final ServerBean server) {
        for (DeploymentBean deploymentBean : operation.getDeploymentlist()) {
            if (deploymentBean.getServerName().equals(server.getName())) {
                return deploymentBean;
            }
        }
        return null;
    }

    /**
     * This function allows to execute the next sub step in the migration
     * execution step
     */
    private void nextExecuteMigration() throws JadortServiceException {
        for (DeploymentBean deployment : operation.getDeploymentlist()) {
            deployment.setDeploymentState(operation.getAimedDeploymentState());
            deployment.setActionState(ActionState.RUNNING);
        }

        switch (operation.getAimedDeploymentState()) {
            case INITIAL:
                operation.setAimedDeploymentState(DeploymentState.UPLOAD_OK);
                operation.setAimedDeploymentProgress(33);
                break;

            case UPLOAD_OK:
                operation.setAimedDeploymentState(DeploymentState.DEPLOY_OK);
                operation.setAimedDeploymentProgress(66);
                break;

            case DEPLOY_OK:
                operation.setAimedDeploymentState(DeploymentState.SET_DEFAULT_OK);
                operation.setAimedDeploymentProgress(100);
                break;

            case SET_DEFAULT_OK:
                operation.setAimedDeploymentState(DeploymentState.UNDEPLOY_OK);
                operation.setAimedDeploymentProgress(50);
                break;

            case UNDEPLOY_OK:
                operation.setAimedDeploymentState(DeploymentState.ERASE_OK);
                operation.setAimedDeploymentProgress(100);
                break;
        }

        mergeOperation();

        for (DeploymentBean deployment : operation.getDeploymentlist()) {
            reachAimedDeploymentState(deployment);
        }
    }

    /**
     * this function allows to execute the previous sub step in the migration
     * execution step
     */
    private void previousExecuteMigration() throws JadortServiceException {
        for (DeploymentBean deployment : operation.getDeploymentlist()) {
            deployment.setDeploymentState(operation.getAimedDeploymentState());
            deployment.setActionState(ActionState.RUNNING);
        }

        switch (operation.getAimedDeploymentState()) {
            case INITIAL:
                break;

            case UPLOAD_OK:
                operation.setAimedDeploymentState(DeploymentState.INITIAL);
                operation.setAimedDeploymentProgress(0);
                break;

            case DEPLOY_OK:
                operation.setAimedDeploymentState(DeploymentState.UPLOAD_OK);
                operation.setAimedDeploymentProgress(33);
                break;

            case SET_DEFAULT_OK:
                operation.setAimedDeploymentState(DeploymentState.DEPLOY_OK);
                operation.setAimedDeploymentProgress(66);
                break;

            case UNDEPLOY_OK:
                operation.setAimedDeploymentState(DeploymentState.SET_DEFAULT_OK);
                operation.setAimedDeploymentProgress(100);
                break;
        }

        for (DeploymentBean deployment : operation.getDeploymentlist()) {
            reachAimedDeploymentState(deployment);
        }
        mergeOperation();
    }

    /**
     * this function allows the deployment to executes the proper task to attend
     * the Aimed DeploymentState this function executes the correspondent task
     * to go from the current deployment state to the Aimed Deployment State.
     *
     * @param deployment - the deployment that will execute the task.
     */
    private void reachAimedDeploymentState(final DeploymentBean deployment) {
        try {
            switch (deployment.getDeploymentState()) {
                case INITIAL:
                    if (operation.getAimedDeploymentState() == DeploymentState.UPLOAD_OK) {
                        // next
                        upload(deployment, operation.getApplication());
                        return;
                    }
                    // no previous
                    break;

                case UPLOAD_OK:
                    if (operation.getAimedDeploymentState() == DeploymentState.DEPLOY_OK) {
                        // next
                        deploy(deployment, deployment.getNewApplication());
                        return;
                    } else if (operation.getAimedDeploymentState() == DeploymentState.INITIAL) {
                        // previous
                        erase(deployment, deployment.getNewApplication());
                        return;
                    }
                    break;

                case DEPLOY_OK:
                    if (operation.getAimedDeploymentState() == DeploymentState.SET_DEFAULT_OK) {
                        // next
                        setAsDefault(deployment, deployment.getNewApplication());
                        return;
                    } else if (operation.getAimedDeploymentState() == DeploymentState.UPLOAD_OK) {
                        // previous
                        undeploy(deployment, deployment.getNewApplication());
                        return;
                    }
                    break;

                case SET_DEFAULT_OK:
                    if (operation.getAimedDeploymentState() == DeploymentState.UNDEPLOY_OK) {
                        // next
                        if (!deployment.hasOldVersion()) {
                            throw new Exception("There's no old version or the old and new versions are the same");
                        } else {
                            undeploy(deployment, deployment.getOldApplication());
                        }
                        return;
                    } else if (operation.getAimedDeploymentState() == DeploymentState.DEPLOY_OK) {
                        // previous
                        if (deployment.getOldApplication() == null) {
                            deployment.appendToLog("There's no old version, ignoring step.\n"
                                    + "This implies that the new [and only] version will keep on being the default one.");
                            deployment.setActionState(ActionState.WAITING);
                            deployment.setDeploymentState(operation.getAimedDeploymentState());
                            deployment.setProgress(operation.getAimedDeploymentProgress());
                        } else {
                            setAsDefault(deployment, deployment.getOldApplication());
                        }
                        return;
                    }
                    break;

                case UNDEPLOY_OK:
                    if (operation.getAimedDeploymentState() == DeploymentState.ERASE_OK) {
                        // next
                        if (!deployment.hasOldVersion()) {
                            throw new Exception("There's no old version or the old and new versions are the same");
                        } else {
                            erase(deployment, deployment.getOldApplication());
                        }
                        return;
                    } else if (operation.getAimedDeploymentState() == DeploymentState.SET_DEFAULT_OK) {
                        // previous
                        deploy(deployment, deployment.getOldApplication());
                        return;
                    }
                    break;
            }
        } catch (Exception e) {
            deployment.appendToLog("Failed executing server action: " + e.getMessage());
            deployment.setActionState(ActionState.FINISHED_ERROR);
            return;
        }

        deployment
                .appendToLog("Don't know what to do: operation.getAimedDeploymentState() is "
                        + operation.getAimedDeploymentState() + ", deployment.getDeploymentState() is "
                        + deployment.getDeploymentState());
        deployment.setActionState(ActionState.FINISHED_ERROR);
        return;
    }

    /**
     * this function allows to upload an application on a server
     *
     * @param deployment - the deployment server on which the application will
     *        be uploaded.
     * @param application - the application to be uploaded.
     * @throws JMSException
     */

    private void upload(final DeploymentBean deployment, final ApplicationBean application) throws JMSException {
        Message message = session.createMessage();
        message.setStringProperty(JadortServiceImpl.ACTION_TYPE, ActionType.UPLOAD.toString());
        message.setIntProperty(JadortServiceImpl.DEPLOYMENT_ID, deployment.getId());
        message.setIntProperty(JadortServiceImpl.APPLICATION_ID, application.getId());
        QueueSender sender = session.createSender(queue);
        sender.send(message);
    }

    /**
     * this function allow to deploy an application on the server
     *
     * @param deployment - the deployment server on which the application will
     *        be deployed
     * @param applicationName - the application name to be deployed
     */
    private void deploy(final DeploymentBean deployment, final String applicationName) throws JMSException,
            JadortServiceException {
        if (applicationName == null) {
            deployment.setDeploymentState(operation.getAimedDeploymentState());
            deployment.setProgress(operation.getAimedDeploymentProgress());
            deployment.setActionState(ActionState.FINISHED_ERROR);
            deployment.appendToLog("The application to deploy is null !");
        } else {
            Message message = session.createMessage();
            message.setStringProperty(JadortServiceImpl.ACTION_TYPE, ActionType.DEPLOY.toString());
            message.setIntProperty(JadortServiceImpl.DEPLOYMENT_ID, deployment.getId());
            message.setStringProperty(JadortServiceImpl.APPLICATION_NAME, applicationName);
            QueueSender sender = session.createSender(queue);
            sender.send(message);
        }
    }

    /**
     * This function allows to activate an application on the the server. it
     * makes the application policy to default.
     *
     * @param deployment - the deployment server on which the new application
     *        will be activated.
     * @param applicationName - the application name
     */
    private void setAsDefault(final DeploymentBean deployment, final String applicationName) throws JMSException,
            JadortServiceException {
        if (applicationName == null) {
            deployment.setDeploymentState(operation.getAimedDeploymentState());
            deployment.setProgress(operation.getAimedDeploymentProgress());
            deployment.setActionState(ActionState.FINISHED_ERROR);
            deployment.appendToLog("The application to set as default is null !");
        } else {
            Message message = session.createMessage();
            message.setStringProperty(JadortServiceImpl.ACTION_TYPE, ActionType.SET_DEFAULT.toString());
            message.setIntProperty(JadortServiceImpl.DEPLOYMENT_ID, deployment.getId());
            message.setStringProperty(JadortServiceImpl.APPLICATION_NAME, applicationName);
            QueueSender sender = session.createSender(queue);
            sender.send(message);
        }
    }

    /**
     * this function allows to undeploy an application on the server
     *
     * @param deployment - the deployment server on which the application will
     *        be undeployed
     * @param applicationName - the application name to undeploy
     */
    private void undeploy(final DeploymentBean deployment, final String applicationName) throws JMSException,
            JadortServiceException {
        if (applicationName == null) {
            deployment.setDeploymentState(operation.getAimedDeploymentState());
            deployment.setProgress(operation.getAimedDeploymentProgress());
            deployment.setActionState(ActionState.FINISHED_ERROR);
            deployment.appendToLog("The application to undeploy is null !");
        } else {
            Message message = session.createMessage();
            message.setStringProperty(JadortServiceImpl.ACTION_TYPE, ActionType.UNDEPLOY.toString());
            message.setIntProperty(JadortServiceImpl.DEPLOYMENT_ID, deployment.getId());
            message.setStringProperty(JadortServiceImpl.APPLICATION_NAME, applicationName);
            QueueSender sender = session.createSender(queue);
            sender.send(message);
        }
    }

    /**
     * this function allow to erase an application on the server
     *
     * @param deployment - the deployment server on which the new application
     *        will be erased
     * @param applicationName - the application name to erase
     */
    private void erase(final DeploymentBean deployment, final String applicationName) throws JMSException, JadortServiceException {
        if (applicationName == null) {
            deployment.setDeploymentState(operation.getAimedDeploymentState());
            deployment.setProgress(operation.getAimedDeploymentProgress());
            deployment.setActionState(ActionState.FINISHED_ERROR);
            deployment.appendToLog("The application to erase is null !");
        } else {
            Message message = session.createMessage();
            message.setStringProperty(JadortServiceImpl.ACTION_TYPE, ActionType.ERASE.toString());
            message.setIntProperty(JadortServiceImpl.DEPLOYMENT_ID, deployment.getId());
            message.setStringProperty(JadortServiceImpl.APPLICATION_NAME, applicationName);
            QueueSender sender = session.createSender(queue);
            sender.send(message);
        }
    }

    /**
     * This function allows to restart a task on a server (the server will
     * restart the last task)
     *
     * @param server - the server on which the task will be restarted
     * @see org.ow2.jasmine.jadort.service.IJadortService#restartServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the restartServermethod can not be called in the current
     *         step.
     *         <li>if the server state is not in error state. (this method can
     *         be called only in Step.EXECUTING_MIGRATION_PART1 and
     *         Step.EXECUTING_MIGRATION_PART2 steps)
     *         </ul>
     */
    public void restartServer(final ServerBean server) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.EXECUTING_MIGRATION_PART1, Step.EXECUTING_MIGRATION_PART2);

            DeploymentBean deployment = getDeployment(server);
            if (deployment.getActionState() != ActionState.FINISHED_ERROR) {
                throw new JadortServiceException("Server state for server '" + server.getName() + "' is "
                        + deployment.getActionState() + ", it should be " + ActionState.FINISHED_ERROR, null);
            }
            deployment.setActionState(ActionState.RUNNING);

            mergeOperation();

            reachAimedDeploymentState(getDeployment(server));
        }
    }

    /**
     * this function forces the server to be ignored and continues this task
     *
     * @param server - the server that will be ignored
     * @see org.ow2.jasmine.jadort.service.IJadortService#ignoreServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the ignoreServer can not be called in the current step.
     *         <li>if the server state is not in error state. (this method can
     *         be called only in EXECUTING_MIGRATION_PART1 and
     *         EXECUTING_MIGRATION_PART2 steps)
     *         </ul>
     */
    public void ignoreServer(final ServerBean server) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.EXECUTING_MIGRATION_PART1, Step.EXECUTING_MIGRATION_PART2);

            DeploymentBean deployment = getDeployment(server);
            if (deployment.getActionState() != ActionState.FINISHED_ERROR) {
                throw new JadortServiceException("Server state for server '" + server.getName() + "' is "
                        + deployment.getActionState() + ", it should be " + ActionState.FINISHED_ERROR, null);
            }

            deployment.appendToLog("Error on the server has been ignored by the user");

            deployment.setActionState(ActionState.WAITING);
            deployment.setDeploymentState(operation.getAimedDeploymentState());

            mergeOperation();
        }
    }

    /**
     * this function forces the server to be ignored and continues this task
     *
     * @param server - the server that will be ignored
     * @see org.ow2.jasmine.jadort.service.IJadortService#abortServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the abortServer can not be called in the current step.
     *         <li>if the server state is not in running state. (this method
     *         can be called only in Step.EXECUTING_MIGRATION_PART1 and
     *         Step.EXECUTING_MIGRATION_PART2 steps)
     *         </ul>
     */
    public void abortServer(final ServerBean server) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.EXECUTING_MIGRATION_PART1, Step.EXECUTING_MIGRATION_PART2);

            DeploymentBean deployment = getDeployment(server);
            if (deployment.getActionState() != ActionState.RUNNING) {
                throw new JadortServiceException("Server state for server '" + server.getName() + "' is "
                        + deployment.getActionState() + ", it should be " + ActionState.RUNNING, null);
            }

            deployment.appendToLog("Action has been aborted by the user");

            deployment.setActionState(ActionState.FINISHED_ERROR);
            deployment.setProgress(operation.getAimedDeploymentProgress());

            mergeOperation();
        }
    }

    /**
     * this function checks if no problem on the server and all problems are
     * resolved on the server.
     *
     * @return true-> OK the problem server is resolved and it can be ignored ,
     *         false -> KO the problem still
     * @param server - the server that will be checked
     * @see JadortServiceImpl#ignoreServer()
     * @see org.ow2.jasmine.jadort.service.IJadortService#checkServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the checkServer can not be called in the current step.
     *         (this method can be called only in Step.EXECUTING_MIGRATION_PART1
     *         and Step.EXECUTING_MIGRATION_PART2 steps)
     *         </ul>
     */
    public boolean checkServer(final ServerBean server) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.EXECUTING_MIGRATION_PART1, Step.EXECUTING_MIGRATION_PART2);

            try {
                boolean result = true;
                DeploymentBean deployment = this.getDeployment(server);
                ServerAction serverAction = ServerAction.getServerAction(deployment.getServer());
                List<ApplicationBean> applications = serverAction.listOfApplications();

                switch (deployment.getDeploymentState()) {
                    case INITIAL:
                        for (ApplicationBean application : applications) {
                            if (isSameApplication(operation.getApplication(), application)) {
                                if (!checkApplication(deployment, application, null, null)) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getNewApplication()), application)) {
                                if (!checkApplication(deployment, application, null, null)) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getOldApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                        }
                        break;

                    case UPLOAD_OK:
                        for (ApplicationBean application : applications) {
                            if (isSameApplication(operation.getApplication(), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_PRESENT, null)) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getNewApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_PRESENT, null)) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getOldApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                        }
                        break;

                    case DEPLOY_OK:
                        for (ApplicationBean application : applications) {
                            if (isSameApplication(operation.getApplication(), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Reserved")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getNewApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Reserved")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getOldApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                        }
                        break;

                    case SET_DEFAULT_OK:
                        for (ApplicationBean application : applications) {
                            if (isSameApplication(operation.getApplication(), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getNewApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getOldApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Disabled")) {
                                    result = false;
                                    break;
                                }
                            }
                        }
                        break;

                    case UNDEPLOY_OK:
                        for (ApplicationBean application : applications) {
                            if (isSameApplication(operation.getApplication(), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getNewApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getOldApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_PRESENT, null)) {
                                    result = false;
                                    break;
                                }
                            }
                        }
                        break;

                    case ERASE_OK:
                        for (ApplicationBean application : applications) {
                            if (isSameApplication(operation.getApplication(), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getNewApplication()), application)) {
                                if (!checkApplication(deployment, application, ServerAction.STATE_DEPLOYED, "Default")) {
                                    result = false;
                                    break;
                                }
                            }
                            if (isSameApplication(serverAction.getApplicationBean(deployment.getOldApplication()), application)) {
                                if (!checkApplication(deployment, application, null, null)) {
                                    result = false;
                                    break;
                                }
                            }
                        }
                        break;
                }

                mergeOperation();

                return result;
            } catch (Exception e) {
                e.printStackTrace();
                throw new JadortServiceException("Cannot check server '" + server + "': " + e.getMessage(), e);
            }
        }
    }

    /**
     * Compares two application, tests if a two applications are equal). The
     * comparison is based on the name,and version of each application. the
     * result is a true the two applications are the same. the result is false
     * if the two applications are different.
     *
     * @param a1 - the first application to be compared.
     * @param a2 - the second application to be compared.
     * @return True the two applications are the same. False if the two
     *         applications are different.
     */
    protected boolean isSameApplication(final ApplicationBean a1, final ApplicationBean a2) {
        if (a1 == null || a2 == null || a1.getName() == null) {
            return false;
        }
        return a1.getName().equals(a2.getName())
                && (a1.getVersion() == a2.getVersion() || a1.getVersion() != null && a1.getVersion().equals(a2.getVersion()));
    }

    /**
     * Checks if an application is already deployed on the server, or if has a
     * bad policy, or if has a bad state
     *
     * @param deployment - deployment of the server on which the application
     *        will be checked.
     * @param application - the application to be checked
     * @param state - the state that the application should has.
     * @param policy - the policy that the application should has.
     * @return true if no problem is found while checking the application false
     *         if a problem is found while checking the application
     */
    protected boolean checkApplication(final DeploymentBean deployment, final ApplicationBean application, final String state,
            final String policy) {
        if (state == null) {
            if (application != null) {
                deployment.appendToLog("Application " + application
                        + " exists (it should not exist), checkServer will return false");
                return false;
            } else {
                return true;
            }
        } else {
            if (state.equals(application.getState())) {
                if (policy == null) {
                    if (application.getPolicy() != null) {
                        deployment.appendToLog("Application " + application + " has policy " + application.getPolicy()
                                + " (it should not have any), checkServer will return false");
                        return false;
                    } else {
                        return true;
                    }
                } else {
                    if (!policy.equals(application.getPolicy())) {
                        deployment.appendToLog("Application " + application + " has policy " + application.getPolicy()
                                + " instead of " + policy + ", checkServer will return false");
                        return false;
                    } else {
                        return true;
                    }
                }
            } else {
                deployment.appendToLog("Application " + application + " has state " + application.getState() + " instead of "
                        + state + ", checkServer will return false");
                return false;
            }
        }
    }

    /**
     * Create a new application
     *
     * @return ApplicationBean - the created ApplicationBean
     * @param url - URL where to fetch the application. Can be local, remote
     *        (HTTP, FTP, ...)
     * @see org.ow2.jasmine.jadort.service.IJadortService#createApplication(java.net.URL)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the createApplication can not be called in the current
     *         step. (this method can be called only in SELECT_APPLICATION step)
     *         </ul>
     */
    public ApplicationBean createApplication(final URL url) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.SELECT_APPLICATION);

            ApplicationBean applicationBean;
            File file;
            if (url.getProtocol().equals("file")) {
                try {
                    file = new File(url.toURI());
                } catch (URISyntaxException e) {
                    throw new JadortServiceException("Cannot create file for URL '" + url + "': " + e.getMessage(), e);
                }
            } else {
                try {
                    URLConnection connection = url.openConnection();
                    InputStream reader = connection.getInputStream();
                    String name = url.getPath();
                    name = name.substring(name.lastIndexOf('/'));
                    file = new File(System.getProperty("java.io.tmpdir") + File.separatorChar + "jadort.temp."
                            + System.currentTimeMillis());
                    file.mkdirs();
                    file = new File(file, name);
                    FileOutputStream writer = new FileOutputStream(file);
                    int read;
                    byte[] buffer = new byte[JadortServiceImpl.BUFFER_SIZE];
                    while ((read = reader.read(buffer)) > 0) {
                        writer.write(buffer, 0, read);
                    }
                    writer.close();
                    reader.close();
                } catch (IOException e) {
                    throw new JadortServiceException("Cannot read from URL: '" + url + "'", null);
                }
            }
            try {
                applicationBean = new ApplicationBean(file);
            } catch (Exception e) {
                throw new JadortServiceException("Cannot create application for URL '" + url + "': " + e.getMessage(), e);
            }

            operation.setApplication(applicationBean);

            mergeOperation();

            return this.operation.getApplication();
        }
    }

    /**
     * Returns the current Step of the current operation
     *
     * @return The current step.
     * @see org.ow2.jasmine.jadort.service.IJadortService#getCurrentStep()
     */
    public Step getCurrentStep() {
        synchronized (this) {
            refreshOperation();

            if (operation == null) {
                return Step.SELECT_OPERATION;
            } else {
                return operation.getCurrentStep();
            }
        }
    }

    /**
     * Gets the list of applications.
     */
    private void getApplications() throws JadortServiceException {
        for (GroupBean groupBean : operation.getTopology().getGroups()) {
            List<ApplicationBean> applications = new ArrayList<ApplicationBean>();
            // Assume all servers in a group have the same applications
            ServerBean serverBean = groupBean.getServers().get(0);
            try {
                for (ApplicationBean applicationBean : ServerAction.getServerAction(serverBean).listOfApplications()) {
                    applications.add(applicationBean);
                }
            } catch (Exception e) {
                throw new JadortServiceException("Failed getting the applications on the servers: " + e.getMessage(), e);
            }
            groupBean.setApplications(applications);
        }
    }

    /**
     * @return true if any deployment has an old version, false otherwise.
     */
    private boolean hasOldVersion() {
        for (DeploymentBean deployment : operation.getDeploymentlist()) {
            if (deployment.hasOldVersion()) {
                return true;
            }
        }
        return false;
    }

    /**
     * Goes to next step, or next task(sub-step)
     *
     * @see JadortServiceImpl#canGoToNextStep()
     * @see org.ow2.jasmine.jadort.service.IJadortService#next()
     * @throws JadortServiceException if it is not allowed to go to the next
     *         step.
     */
    public void next() throws JadortServiceException {
        synchronized (this) {
            if (!canGoToNextStep()) {
                throw new JadortServiceException("You are not allowed to go to the next step!", null);
            }

            // We don't need to call refreshOperation() as canGoToNextStep() has
            // already done it

            if (getCurrentStep() == OperationStateBean.Step.EXECUTING_MIGRATION_PART1) {
                if (operation.getAimedDeploymentState() == DeploymentState.SET_DEFAULT_OK) {
                    if (hasOldVersion()) {
                        // If there's an old version present,
                        // prepare for Step.EXECUTING_MIGRATION_PART2
                        for (DeploymentBean deploymentServer : operation.getDeploymentlist()) {
                            deploymentServer.setProgress(0);
                        }
                    } else {
                        // Else, prepare for Step.FINISHED
                        for (DeploymentBean deploymentServer : operation.getDeploymentlist()) {
                            deploymentServer.setActionState(ActionState.FINISHED_OK);
                        }
                        operation.setCurrentStep(Step.EXECUTING_MIGRATION_PART2);
                    }
                    this.goToNextStep();
                } else {
                    this.nextExecuteMigration();
                }
            } else if (getCurrentStep() == OperationStateBean.Step.EXECUTING_MIGRATION_PART2) {
                if (operation.getAimedDeploymentState() == DeploymentState.ERASE_OK) {
                    for (DeploymentBean deploymentServer : operation.getDeploymentlist()) {
                        deploymentServer.setActionState(ActionState.FINISHED_OK);
                    }
                    this.goToNextStep();
                } else {
                    this.nextExecuteMigration();
                }
            } else {
                this.goToNextStep();
            }
        }
    }

    /**
     * go the next step
     */
    private void goToNextStep() throws JadortServiceException {
        switch (operation.getCurrentStep()) {
            case SELECT_OPERATION:
                operation.setCurrentStep(Step.INITIALIZE_TOPOLOGY);
                break;

            case INITIALIZE_TOPOLOGY:
                operation.setCurrentStep(Step.SELECT_GROUP);
                getApplications();
                break;

            case SELECT_GROUP:
                operation.setCurrentStep(Step.SELECT_APPLICATION);
                getApplications();
                break;

            case SELECT_APPLICATION:
                operation.setCurrentStep(Step.EXECUTING_MIGRATION_PART1);
                break;

            case EXECUTING_MIGRATION_PART1:
                operation.setCurrentStep(Step.EXECUTING_MIGRATION_PART2);
                break;

            case EXECUTING_MIGRATION_PART2:
                operation.setCurrentStep(Step.FINISHED);
                break;
        }

        mergeOperation();
    }

    /**
     * go the previous step
     */
    private void goToPreviousStep() throws JadortServiceException {
        switch (operation.getCurrentStep()) {
            case INITIALIZE_TOPOLOGY:
                operation.setCurrentStep(Step.SELECT_OPERATION);
                break;

            case SELECT_GROUP:
                operation.setCurrentStep(Step.INITIALIZE_TOPOLOGY);
                break;

            case SELECT_APPLICATION:
                operation.setCurrentStep(Step.SELECT_GROUP);
                getApplications();
                break;

            case EXECUTING_MIGRATION_PART1:
                operation.setCurrentStep(Step.SELECT_APPLICATION);
                getApplications();
                break;

            case EXECUTING_MIGRATION_PART2:
                operation.setCurrentStep(Step.EXECUTING_MIGRATION_PART1);
                break;

            case FINISHED:
                if (hasOldVersion()) {
                    operation.setCurrentStep(Step.EXECUTING_MIGRATION_PART2);
                } else {
                    operation.setCurrentStep(Step.EXECUTING_MIGRATION_PART1);
                }
                break;
        }

        mergeOperation();
    }

    /**
     * Goes to previous step, or previous task(sub-step)
     *
     * @see JadortServiceImpl#canGoToPreviousStep()
     * @see org.ow2.jasmine.jadort.service.IJadortService#previous()
     * @throws JadortServiceException
     *         <ul>
     *         <li>if it is not allowed to go to the previous step.
     *         </ul>
     */
    public void previous() throws JadortServiceException {
        synchronized (this) {
            if (!canGoToPreviousStep()) {
                throw new JadortServiceException("You are not allowed to go to the previous step!", null);
            }

            // We don't need to call refreshOperation() as canGoToPreviousStep()
            // has already done it

            if (this.getCurrentStep() == OperationStateBean.Step.SELECT_GROUP) {
                TopologyBean oldTopology = operation.getTopology();
                operation.setTopology(null);
                this.goToPreviousStep();

                em.remove(oldTopology);
                em.flush();
            } else if (this.getCurrentStep() == OperationStateBean.Step.SELECT_APPLICATION) {
                // Don't remove the oldSelectedGroup, as the GroupBeans are
                // created at topology initialization and no copy is done when
                // selecting group). Remove oldDeploymentlist since they're
                // useless now.
                //
                // Remember to copy the oldDeploymentlist array to avoid
                // LazyInitializationException
                List<DeploymentBean> oldDeploymentlist = new ArrayList<DeploymentBean>(operation.getDeploymentlist());
                operation.setDeploymentlist(null);
                operation.setSelectedGroup(null);
                this.goToPreviousStep();

                for (DeploymentBean deploymentBean : oldDeploymentlist) {
                    em.remove(deploymentBean);
                }
                em.flush();
            } else if (this.getCurrentStep() == OperationStateBean.Step.EXECUTING_MIGRATION_PART1) {
                if (operation.getAimedDeploymentState() == DeploymentState.INITIAL) {
                    ApplicationBean oldApplication = operation.getApplication();
                    operation.setApplication(null);
                    this.goToPreviousStep();

                    em.remove(oldApplication);
                    em.flush();
                } else {
                    this.previousExecuteMigration();
                }
            } else if (this.getCurrentStep() == OperationStateBean.Step.EXECUTING_MIGRATION_PART2) {
                if (operation.getAimedDeploymentState() == DeploymentState.SET_DEFAULT_OK) {
                    for (DeploymentBean deploymentServer : operation.getDeploymentlist()) {
                        deploymentServer.setProgress(100);
                    }
                    this.goToPreviousStep();
                } else {
                    this.previousExecuteMigration();
                }
            } else if (this.getCurrentStep() == OperationStateBean.Step.FINISHED) {
                for (DeploymentBean deployment : operation.getDeploymentlist()) {
                    deployment.setActionState(ActionState.WAITING);
                }
                this.goToPreviousStep();
            } else {
                throw new JadortServiceException("No previous step for " + this.getCurrentStep(), null);
            }
        }
    }

    /**
     * Tests if it is allowed to go to the next step
     *
     * @return true id calling {@link JadortServiceImpl#next()} is allowed,
     *         false otherwise.
     * @see org.ow2.jasmine.jadort.service.IJadortService#canGoToNextStep()
     */
    public boolean canGoToNextStep() {
        synchronized (this) {
            refreshOperation();

            switch (this.getCurrentStep()) {
                case SELECT_OPERATION:
                    return operation != null;

                case INITIALIZE_TOPOLOGY:
                    return operation.getTopology() != null;

                case SELECT_GROUP:
                    return operation.getSelectedGroup() != null;

                case SELECT_APPLICATION:
                    return operation.getApplication() != null;

                case EXECUTING_MIGRATION_PART1:
                case EXECUTING_MIGRATION_PART2:
                    for (DeploymentBean deployment : operation.getDeploymentlist()) {
                        if (deployment.getActionState() != ActionState.WAITING) {
                            return false;
                        }
                    }
                    return true;

                case FINISHED:
                default:
                    return false;
            }
        }
    }

    /**
     * Tests if it is allowed to go to the previous step
     *
     * @return true if it is allowed to go to the previous step
     *         {@link JadortServiceImpl#previous()} , false otherwise.
     * @see org.ow2.jasmine.jadort.service.IJadortService#canGoToPreviousStep()
     */
    public boolean canGoToPreviousStep() {
        synchronized (this) {
            refreshOperation();
            switch (this.getCurrentStep()) {
                case SELECT_OPERATION:
                    return false;

                case INITIALIZE_TOPOLOGY:
                    return false;

                case SELECT_GROUP:
                    return true;

                case SELECT_APPLICATION:
                    return true;

                case EXECUTING_MIGRATION_PART1:
                case EXECUTING_MIGRATION_PART2:
                    if (operation.getAimedDeploymentState() == DeploymentState.ERASE_OK) {
                        return false;
                    }
                    int progress = -1;
                    for (DeploymentBean deployment : operation.getDeploymentlist()) {
                        if (deployment.getActionState() != ActionState.WAITING
                                && deployment.getActionState() != ActionState.FINISHED_ERROR) {
                            return false;
                        }
                        if (progress == -1) {
                            progress = deployment.getProgress();
                            continue;
                        }
                        if (deployment.getProgress() != progress) {
                            return false;
                        }
                    }
                    return true;

                case FINISHED:
                    return true;

                default:
                    return false;
            }
        }
    }

    /**
     * Select an Operation from the operation list
     *
     * @param operation - the operation to be selected
     * @see org.ow2.jasmine.jadort.service.IJadortService#selectOperation(java.lang.String)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the selectOperation can not be called in the current
     *         step. (this method can be called only in SELECT_OPERATION step)
     *         </ul>
     */
    public void selectOperation(final OperationStateBean selectedOperation) throws JadortServiceException {
        synchronized (this) {
            refreshOperation();
            checkStep(Step.SELECT_OPERATION);

            operation = em.find(OperationStateBean.class, selectedOperation.getId());
            if (getCurrentStep() == Step.SELECT_GROUP) {
                getApplications();
            }
        }
    }

    /**
     * Create a new operation
     *
     * @param newOperationName - the name of the new operation that will be
     *        created
     * @see org.ow2.jasmine.jadort.service.IJadortService#createNewOperation(java.lang.String)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the selectOperation can not be called in the current
     *         step. (this method can be called only in SELECT_OPERATION step)
     *         </ul>
     */
    public void createNewOperation(final String newOperationName) throws JadortServiceException {
        synchronized (this) {
            checkStep(Step.SELECT_OPERATION);

            operation = new OperationStateBean();
            operation.setName(newOperationName);
            operation.setDate(new Date());
            em.persist(operation);
        }
    }

    /**
     * Delete an Operation from the operation list
     *
     * @param operation - the operation to be deleted
     * @see org.ow2.jasmine.jadort.service.IJadortService#deleteOperation(org.ow2.jasmine.jadort.service.entities.OperationStateBean)
     * @throws JadortServiceException
     *         <ul>
     *         <li>if the selectOperation can not be called in the current
     *         step. (this method can be called only in SELECT_OPERATION step)
     *         </ul>
     */
    public void deleteOperation(final OperationStateBean selectedOperation) throws JadortServiceException {
        synchronized (this) {
            checkStep(Step.SELECT_OPERATION);

            OperationStateBean toRemove = em.find(OperationStateBean.class, selectedOperation.getId());
            em.remove(toRemove);
            em.flush();

            if (operation != null && selectedOperation.getId() == operation.getId()) {
                System.out.println("Warning: deleted current operation, setting operation to null");
                operation = null;
            }
        }
    }

    /**
     * This method checks if an method(or command) can be called in the current.
     *
     * @param steps :the list of steps where the calling method(the methods
     *        where checkstep is called) can be called. throws
     *        JadortServiceException
     *        <ul>
     *        <li>if it is not allowed to call the method in the current step.
     *        </ul>
     */
    private void checkStep(final Step... steps) throws JadortServiceException {
        String possibleSteps = null;
        Step currentStep = getCurrentStep();

        for (Step step : steps) {
            if (currentStep == step) {
                return;
            } else {
                if (possibleSteps == null) {
                    possibleSteps = step.toString();
                } else {
                    possibleSteps += " or " + step.toString();
                }
            }
        }

        throw new JadortServiceException("That method should only be called when step is " + possibleSteps
                + ". Currently, step is " + currentStep + ".", null);
    }

    /**
     * This method refreshes {@link JadortServiceImpl#operation} and should be
     * called when entering any method that uses datum that can be modified in
     * another thread than the stateful bean's. This is the case when taking any
     * deployment action: the {@link OperationStateBean} gets updated by the
     * {@link JadortServiceJMSActions} handler in an asynchronous way.
     */
    private void refreshOperation() {
        if (operation != null) {
            operation = em.find(OperationStateBean.class, operation.getId());
        }
    }

    /**
     * This method merges {@link JadortServiceImpl#operation} onto the DB.
     */
    private void mergeOperation() {
        if (operation != null) {
            operation = em.merge(operation);
            em.flush();
        }
    }
}
