/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JadortServiceJMSActions.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.implementation;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.ow2.jasmine.jadort.service.action.ServerAction;
import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean.ActionState;

/**
 * JadortServiceJMSActions class is the Message Driven Bean that allows the execution of
 * deployment tasks(actions) in the migration execution step.
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */

@MessageDriven(activationConfig = {
        @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
        @ActivationConfigProperty(propertyName = "destination", propertyValue = JadortServiceImpl.QUEUE_NAME) })
public class JadortServiceJMSActions implements MessageListener {

    @PersistenceContext
    protected EntityManager em;

    /*
     * (non-Javadoc)
     * 
     * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
     */
    public void onMessage(final Message message) {
        try {
            process(message);
        } catch (Exception e) {
            System.out.println("Failed processing JMS message '" + message + "': " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * processes the JMS message. Execute the deployment actions.
     * 
     * @param message
     * @throws JMSException
     */
    private void process(final Message message) throws JMSException {
        try {
            // Read common message parameters
            Integer deploymentId = message.getIntProperty(JadortServiceImpl.DEPLOYMENT_ID);
            DeploymentBean deployment = em.find(DeploymentBean.class, deploymentId);
            JadortServiceImpl.ActionType actionType = JadortServiceImpl.ActionType.valueOf(message
                    .getStringProperty(JadortServiceImpl.ACTION_TYPE));

            switch (actionType) {
                case UPLOAD:
                    Integer applicationId = message.getIntProperty(JadortServiceImpl.APPLICATION_ID);
                    ApplicationBean application = em.find(ApplicationBean.class, applicationId);

                    upload(deployment, application);
                    break;

                case DEPLOY:
                    String deployApplicationName = message.getStringProperty(JadortServiceImpl.APPLICATION_NAME);

                    deploy(deployment, deployApplicationName);
                    break;

                case SET_DEFAULT:
                    String setDefaultApplicationName = message.getStringProperty(JadortServiceImpl.APPLICATION_NAME);

                    setAsDefault(deployment, setDefaultApplicationName);
                    break;

                case UNDEPLOY:
                    String undeployApplicationName = message.getStringProperty(JadortServiceImpl.APPLICATION_NAME);

                    undeploy(deployment, undeployApplicationName);
                    break;

                case ERASE:
                    String eraseApplicationName = message.getStringProperty(JadortServiceImpl.APPLICATION_NAME);

                    erase(deployment, eraseApplicationName);
                    break;

                default:
                    deployment.setActionState(ActionState.FINISHED_ERROR);
                    deployment.appendToLog("Unknown action type: " + actionType);
                    break;
            }

            em.merge(deployment);
        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

    /**
     * this function allow to deploy an application on the server
     * 
     * @param deployment : the deployment server on which the application will
     *        be deployed
     * @param applicationName : the application name to be deployed
     */
    private void deploy(final DeploymentBean deployment, final String applicationName) throws JMSException {
        OutputStream stackTrace = null;
        ServerAction serverAction = ServerAction.getServerAction(deployment.getServer());

        try {
            serverAction.deploy(applicationName);
        } catch (Exception e) {
            stackTrace = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(stackTrace));
        }

        deployment.appendToLog(serverAction.flushLog());
        if (stackTrace != null) {
            deployment.appendToLog(stackTrace.toString());
        }

        // Check for both == and .equals to avoid
        // NullPointerExceptions
        if (applicationName == deployment.getOldApplication() || applicationName.equals(deployment.getOldApplication())) {
            deployment.setProgress(0);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.SET_DEFAULT_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        } else {
            deployment.setProgress(66);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.DEPLOY_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        }
    }

    /**
     * this function allows to upload an application on a server
     * 
     * @param deployment - the deployment server on which the application will
     *        be uploaded.
     * @param application - the application to be uploaded.
     * @throws JMSException
     */

    private void upload(final DeploymentBean deployment, final ApplicationBean application) {
        OutputStream stackTrace = null;
        ServerAction serverAction = ServerAction.getServerAction(deployment.getServer());

        String newApplication = null;
        try {
            newApplication = serverAction.upload(application);
        } catch (Exception e) {
            stackTrace = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(stackTrace));
        }

        if (newApplication != null) {
            deployment.setNewApplication(newApplication);
        }
        deployment.appendToLog(serverAction.flushLog());
        if (stackTrace != null) {
            deployment.appendToLog(stackTrace.toString());
        }

        deployment.setProgress(33);

        if (stackTrace == null) {
            deployment.setActionState(ActionState.WAITING);
            deployment.setDeploymentState(DeploymentBean.DeploymentState.UPLOAD_OK);
        } else {
            deployment.setActionState(ActionState.FINISHED_ERROR);
        }
    }

    /**
     * This function allows to activate an application on the the server. it
     * makes the application policy to default.
     * 
     * @param deployment - the deployment server on which the new application
     *        will be activated.
     * @param applicationName : the application name
     */
    private void setAsDefault(final DeploymentBean deployment, final String applicationName) {
        OutputStream stackTrace = null;
        ServerAction serverAction = ServerAction.getServerAction(deployment.getServer());

        String previousDefault = null;
        try {
            previousDefault = serverAction.setDefault(applicationName);
        } catch (Exception e) {
            stackTrace = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(stackTrace));
        }

        if (previousDefault != null && deployment.getOldApplication() == null) {
            deployment.setOldApplication(previousDefault);
        }

        deployment.appendToLog(serverAction.flushLog());
        if (stackTrace != null) {
            deployment.appendToLog(stackTrace.toString());
        }

        // Check for both == and .equals to avoid
        // NullPointerExceptions
        if (applicationName == deployment.getOldApplication() || applicationName.equals(deployment.getOldApplication())) {
            deployment.setProgress(66);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.DEPLOY_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        } else {
            deployment.setProgress(100);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.SET_DEFAULT_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        }
    }

    /**
     * this function allows to undeploy an application on the server
     * 
     * @param deployment : the deployment server on which the application will
     *        be undeployed
     * @param applicationName : the application name to be undeployed
     */
    private void undeploy(final DeploymentBean deployment, final String applicationName) {
        OutputStream stackTrace = null;
        ServerAction serverAction = ServerAction.getServerAction(deployment.getServer());

        try {
            serverAction.undeploy(applicationName);
        } catch (Exception e) {
            stackTrace = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(stackTrace));
        }

        deployment.appendToLog(serverAction.flushLog());
        if (stackTrace != null) {
            deployment.appendToLog(stackTrace.toString());
        }

        // Check for both == and .equals to avoid
        // NullPointerExceptions
        if (applicationName == deployment.getOldApplication() || applicationName.equals(deployment.getOldApplication())) {
            deployment.setProgress(50);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.UNDEPLOY_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        } else {
            deployment.setProgress(33);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.UPLOAD_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        }
    }

    /**
     * this function allow to erase an application on the server
     * 
     * @param deployment : the deployment server on which the new application
     *        will be erased
     * @param applicationName : the application name to be erased
     */
    private void erase(final DeploymentBean deployment, final String applicationName) {
        OutputStream stackTrace = null;
        ServerAction serverAction = ServerAction.getServerAction(deployment.getServer());

        try {
            serverAction.erase(applicationName);
        } catch (Exception e) {
            stackTrace = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(stackTrace));
        }

        deployment.appendToLog(serverAction.flushLog());
        if (stackTrace != null) {
            deployment.appendToLog(stackTrace.toString());
        }

        // Check for both == and .equals to avoid
        // NullPointerExceptions
        if (applicationName == deployment.getOldApplication() || applicationName.equals(deployment.getOldApplication())) {
            deployment.setProgress(100);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.ERASE_OK);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        } else {
            deployment.setProgress(0);
            if (stackTrace == null) {
                deployment.setActionState(ActionState.WAITING);
                deployment.setDeploymentState(DeploymentBean.DeploymentState.INITIAL);
            } else {
                deployment.setActionState(ActionState.FINISHED_ERROR);
            }
        }
    }

}
