/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JadortServiceClient.java 1503 2008-06-23 13:36:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.client;

import java.io.File;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.ow2.jasmine.jadort.service.IJadortService;
import org.ow2.jasmine.jadort.service.JadortServiceException;
import org.ow2.jasmine.jadort.service.entities.ApplicationBean;
import org.ow2.jasmine.jadort.service.entities.DeploymentBean;
import org.ow2.jasmine.jadort.service.entities.GroupBean;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean;
import org.ow2.jasmine.jadort.service.entities.ServerBean;
import org.ow2.jasmine.jadort.service.entities.OperationStateBean.Step;

/**
 * This class handles the communication between the Web client and the
 * Jadort Service stateful bean.
 *
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public class JadortServiceClient implements Serializable {

    private static final long serialVersionUID = 3148455079928943579L;

    private IJadortService jadortService;

    private static final Map<OperationStateBean.Step, Integer> Step_PageIndex = new Hashtable<Step, Integer>();
    static {
        JadortServiceClient.Step_PageIndex.put(Step.SELECT_OPERATION, 0);
        JadortServiceClient.Step_PageIndex.put(Step.INITIALIZE_TOPOLOGY, 1);
        JadortServiceClient.Step_PageIndex.put(Step.SELECT_GROUP, 2);
        JadortServiceClient.Step_PageIndex.put(Step.SELECT_APPLICATION, 3);
        JadortServiceClient.Step_PageIndex.put(Step.EXECUTING_MIGRATION_PART1, 4);
        JadortServiceClient.Step_PageIndex.put(Step.EXECUTING_MIGRATION_PART2, 5);
        JadortServiceClient.Step_PageIndex.put(Step.FINISHED, 6);
    }

    private JadortServiceExceptionWithInfo analyzeException(final Exception e) {
        // Check for JadortServiceException
        for (Throwable cause = e; cause != null; cause = cause.getCause()) {
            if (cause instanceof JadortServiceException) {
                return new JadortServiceExceptionWithInfo(cause.getMessage());
            }
        }

        // If not JadortServiceException, the exception has been thrown by
        // the container. Output the stack trace, since this probably is a bug.
        Throwable cause = e.getCause();
        if (cause == null) {
            cause = e;
        }
        String message = "Container exception in the JadortService client: " + cause.getMessage();
        synchronized (this) {
            System.out.println(message);
            e.printStackTrace();
        }
        return new JadortServiceExceptionWithInfo(message);
    }

    public void initialize() throws JadortServiceExceptionWithInfo {
        try {
            Context context = new InitialContext();
            jadortService = (IJadortService) context.lookup("JadortConsoleStatefulBean");
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#canGoToNextStep()
     */
    public boolean canGoToNextStep() throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.canGoToNextStep();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#canGoToPreviousStep()
     */
    public boolean canGoToPreviousStep() throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.canGoToPreviousStep();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#checkServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     */
    public boolean checkServer(final ServerBean server) throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.checkServer(server);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    public ApplicationBean createApplication(final String urlOrFile) throws JadortServiceExceptionWithInfo {
        try {
            URL url;
            try {
                url = new URL(urlOrFile);
            } catch (MalformedURLException e) {
                // urlOrFile is not a URL, it is a file
                //
                // Use File.oURI().toURL() and not File.toURL() since
                // File.toURL() doesn't properly escape the URL.
                url = new File(urlOrFile).toURI().toURL();
            }
            return jadortService.createApplication(url);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#ignoreServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     */
    public void ignoreServer(final ServerBean server) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.ignoreServer(server);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#getDeployments()
     */
    public List<DeploymentBean> getDeployments() throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.getDeployments();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#getGroups()
     */
    public List<GroupBean> getGroups() throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.getGroups();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#getPageIndex()
     */
    public int getPageIndex() throws JadortServiceExceptionWithInfo {
        try {
            return JadortServiceClient.Step_PageIndex.get(jadortService.getCurrentStep());
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#getSelectedGroup()
     */
    public GroupBean getSelectedGroup() throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.getSelectedGroup();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#loadTopology(java.io.File)
     */
    public void loadTopology(final String xmlTopoFile) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.loadTopology(new File(xmlTopoFile));
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#next()
     */
    public int next() throws JadortServiceExceptionWithInfo {
        try {
            jadortService.next();
            return getPageIndex();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#previous()
     */
    public int previous() throws JadortServiceExceptionWithInfo {
        try {
            jadortService.previous();
            return getPageIndex();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#restartServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     */
    public void restartServer(final ServerBean server) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.restartServer(server);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#cancelServer(org.ow2.jasmine.jadort.service.topology.ServerBean)
     */
    public void abortServer(final ServerBean server) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.abortServer(server);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#selectGroup(org.ow2.jasmine.jadort.service.entities.GroupBean)
     */
    public void selectGroup(final GroupBean selectedGroup) throws JadortServiceExceptionWithInfo {
        try {
            // Only send out a subset to avoid ClassCastException
            GroupBean selectedGroupCopy = new GroupBean();
            selectedGroupCopy.setId(selectedGroup.getId());
            selectedGroupCopy.setName(selectedGroup.getName());
            selectedGroupCopy.setType(selectedGroup.getType());
            jadortService.selectGroup(selectedGroupCopy);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#createNewOperation(java.lang.String)
     */
    public OperationStateBean createNewOperation(final String newOperationName) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.createNewOperation(newOperationName);
            return jadortService.getCurrentOperation();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#selectOperation(java.lang.String)
     */
    public List<OperationStateBean> getOperationsList() throws JadortServiceExceptionWithInfo {
        try {
            return jadortService.getOperationsList();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#selectOperation(java.lang.String)
     */
    public OperationStateBean selectOperation(final OperationStateBean operation) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.selectOperation(operation);
            return jadortService.getCurrentOperation();
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jadort.service.IJadortService#selectOperation(java.lang.String)
     */
    public void deleteOperation(final OperationStateBean operation) throws JadortServiceExceptionWithInfo {
        try {
            jadortService.deleteOperation(operation);
        } catch (Exception e) {
            throw analyzeException(e);
        }
    }

}
